package com.sillycycle.bagleyd.util;

/*
 * @(#)Graphics.java
 *
 * Copyright 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Path;

/**
 * The <code>Graphics</code> class does all the low level graphics.
 * This is make the android graphics look more like awt.  Not sure
 * why they chose floating point for some operations.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/abacus.html
 */

public class Graphics {
	Canvas canvas;
	Paint paint;

	public Graphics(Canvas canvasInstance, Paint paintInstance) {
		this.canvas = canvasInstance;
		this.paint = paintInstance;
	}

	public void setColor(int color) {
		paint.setColor(color);
	}

	public int getColor() {
		return paint.getColor();
	}

	public void drawLine(int x1, int y1, int x2, int y2) {

		canvas.drawLine(x1, y1, x2, y2, paint);
	}

	public void drawPoly(Point[] points) {
		Path polyPath = new Path();
		paint.setStyle(Paint.Style.FILL);
		polyPath.moveTo(points[0].x, points[0].y);
                for (Point point : points) {
                        polyPath.lineTo(point.x, point.y);
                }
		polyPath.lineTo(points[0].x, points[0].y);
		canvas.drawPath(polyPath, paint);
	}

	public void drawCircle(int diameter, int x, int y) {
		float xCenter = x, yCenter = y;

		if (diameter <= 0)
			return;
		paint.setStyle(Paint.Style.STROKE);
		int radius = diameter / 2;
		if (diameter % 2 == 0) {
			canvas.drawCircle(xCenter, yCenter, radius, paint);
			canvas.drawCircle(xCenter + 1, yCenter, radius, paint);
			canvas.drawCircle(xCenter, yCenter + 1, radius, paint);
			canvas.drawCircle(xCenter + 1, yCenter + 1, radius, paint);
		} else {
			canvas.drawCircle(xCenter, yCenter, radius + 1, paint);
		}
	}

	public void fillCircle(int diameter, int x, int y) {
		if (diameter <= 0)
			return;
		paint.setStyle(Paint.Style.FILL);
		canvas.drawCircle(x, y,
			(diameter) / 2, paint);
	}

	public void fillPolygon(int fillColor, int lineColor,
			Point[] list) {
		/*int[] tempListX = new int[n];
		int[] tempListY = new int[n];

		for (int i = 0; i < n; i++) {
			tempListX[i] = list[i].x;
			tempListY[i] = list[i].y;
		}*/
		paint.setColor(fillColor);
		paint.setStyle(Paint.Style.FILL);
		drawPoly(list);
		paint.setColor(lineColor);
		paint.setStyle(Paint.Style.STROKE);
		drawPoly(list);
	}

	public void drawRect(int i, int j, int w, int h) {
		paint.setStyle(Paint.Style.STROKE);
		canvas.drawRect(i, j, i + w, j + h, paint);
	}

	public void fillRect(int i, int j, int w, int h) {
		paint.setStyle(Paint.Style.FILL);
		canvas.drawRect(i, j, i + w, j + h, paint);
	}

	public void fillRectClipX(int dx, int dy, int sx, int sy,
			int ox, int wox, int wsx) {
		int nox = ox, nsx = sx;

		if (ox + sx < wox || ox > wox + wsx || wsx <= 0)
			return;
		if (nox < wox) {
			nox = wox;
			nsx = sx - wox + ox;
		}
		if (nox + nsx > wox + wsx) {
			nsx = wsx + wox - nox;
		}
		paint.setStyle(Paint.Style.FILL);
		canvas.drawRect(dx + nox, dy, dx + nox + nsx, dy + sy, paint);
	}

	public void fillRectClipY(int dx, int dy, int sx, int sy,
			int oy,int woy, int wsy) {
		int noy = oy, nsy = sy;

		if (oy + sy < woy || oy > woy + wsy || wsy <= 0)
			return;
		if (noy < woy) {
			noy = woy;
			nsy = sy - woy + oy;
		}
		if (noy + nsy > woy + wsy) {
			nsy = wsy + woy - noy;
		}
		paint.setStyle(Paint.Style.FILL);
		canvas.drawRect(dx, dy + noy, dx + sx, dy + noy + nsy, paint);
	}
}
