package com.sillycycle.bagleyd.welltris;

/*
 * @(#)ThingView.java
 *
 * Copyright 1993 - 2016  David A. Bagley, bagleyd AT verizon.net
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995 Q. Alex Zhao, azhao AT cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.util.Random;

import com.sillycycle.bagleyd.util.Colour;

import android.graphics.Canvas;
import android.graphics.Point;

/**
 * The <code>Thing</code> class keeps track of the falling shapes.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/altris.html
 */

public class ThingView {
	//int	     units, polyNumber;
	//Point	     position = new Point(0, 0);
	//int	     size, colorNumber, direction;
	//int	     randomRotation, randomReflection;
	//int	     randomNumber;
	int	     background;
	//boolean	 bonus;

	//private GameView gameView;
	//private Bitmap[] bmps;
	int boxSize;
	int boxWidth = boxSize, boxHeight = boxSize;
	int baseWidth;
	Point fieldOffset = new Point(2, 2);
	int diameter = WelltrisInterface.DEFAULT_WIDTH;
	int depth = WelltrisInterface.DEFAULT_DEPTH;
	int perimeter = WelltrisInterface.MAX_SIDES * diameter;
	private Thing curThing;
	private Spec curSpec = new Spec(WelltrisInterface.DEFAULT_UNITS);
	//private PolyominoImage polyominoImage = new PolyominoImage();
	private PolyMode[] tris; //[WelltrisInterface.MAX_UNITS - MIN_UNITS + 1];
	private int[] frozenWall = new int[WelltrisInterface.MAX_SIDES];
	private Field[][] wall = new Field[depth + diameter][WelltrisInterface.MAX_SIDES * diameter];
	private Field[][] base = new Field[diameter][diameter];
	Random generator;
	private boolean suddenAppear = false;
	private boolean grid = true;
	//PolyominoImage polyImage = null;

	public ThingView(/*GameView gameView, Bitmap[] bmps*/) {
		/*this.gameView = gameView;
		this.bmps = bmps;*/
		curThing = new Thing();
		generator = new Random(System.nanoTime());
		setCurThing();
		for (int j = 0; j < depth + diameter; j++) {
			for (int i = 0; i < perimeter; i++) {
				wall[j][i] = new Field();
				wall[j][i].pmid = -1;
				wall[j][i].cid = 0;
			}
		}
		for (int j = 0; j < diameter; j++) {
			for (int i = 0; i < diameter; i++) {
				base[j][i] = new Field();
				base[j][i].pmid = -1;
				base[j][i].cid = 0;
			}
		}

		unfreezeWalls();
	}

	public void setBoxSize(int size) {
		boxWidth = boxHeight = boxSize = size;
		baseWidth = boxWidth * diameter;
	}

	public int getBoxSize() {
		return boxSize;
	}

	public int getFirstX() {
		if (tris == null)
			System.out.println("tris = null");
		if (tris[curThing.units].poly == null)
			System.out.println("tris.poly = null");
		if (tris[curThing.units].poly[curThing.polyNumber].shape == null)
			System.out.println("tris.poly.shape = null");
		for (int i = 0; i < curThing.size; i++)
			for (int j = 0; j < curThing.size; j++) {

				 if (tris[curThing.units].poly[curThing.polyNumber].shape[j][i] != 0) {
					 return i;
				 }
			}
		return 0;
	}

	public int getLastX() {
		for (int i = curThing.size - 1; i >= 0; i--)
			for (int j = 0; j < curThing.size; j++) {
				 if (tris[curThing.units].poly[curThing.polyNumber].shape[j][i] != 0) {
					 return i;
				 }
			}
		return 0;
	}
	public int getFirstY() {
		for (int j = 0; j < curThing.size; j++)
			for (int i = 0; i < curThing.size; i++) {
				 if (tris[curThing.units].poly[curThing.polyNumber].shape[j][i] != 0) {
					 return j;
				 }
			}
		return 0;
	}

	public int getLastY() {
		for (int j = curThing.size - 1; j >= 0; j--)
			for (int i = 0; i < curThing.size; i++) {
				 if (tris[curThing.units].poly[curThing.polyNumber].shape[j][i] != 0) {
					 return j;
				 }
			}
		return 0;
	}

	public void setOffset(int offsetX, int offsetY) {
		fieldOffset.x = offsetX;
		fieldOffset.y = offsetY;
	}

	public int getOffsetX() {
		return fieldOffset.x;
	}

	public int getOffsetY() {
		return fieldOffset.y;
	}

	public void setPosition(int x, int y) {
		curThing.position.x = x;
		curThing.position.y = y;
	}

	public void setX(int x) {
		curThing.position.x = x;
	}

	public void setY(int y) {
		curThing.position.y = y;
	}

	public int getX() {
		return curThing.position.x;
	}

	public int getY() {
		return curThing.position.y;
	}

	public final void setCurThing() {
		tris = new PolyMode[WelltrisInterface.MAX_UNITS -
			WelltrisInterface.MIN_UNITS + 1];
		for (int i = 0; i < (WelltrisInterface.MAX_UNITS -
				WelltrisInterface.MIN_UNITS + 1); i++)
			tris[i] = new PolyMode(WelltrisInterface.MAX_MODES,
				WelltrisInterface.MAX_TYPES,
				WelltrisInterface.MAX_START_POLYOMINOES,
				WelltrisInterface.MAX_POLYOMINOES,
				WelltrisInterface.MAX_UNITS, WelltrisInterface.MAX_UNITS);
		readPolyominoes();
		setNextThing();
	}

	public void setNextThing() {
		curThing.setRandomNumber(Math.abs(generator.nextInt()));
		curThing.setRotation(generator.nextInt(WelltrisInterface.MAX_SIDES));
		curThing.setReflection(generator.nextInt(2));
		curThing.setBonus(curSpec.bonus);

		int diagonal = (curSpec.diagonal) ? 1 : 0;
		int turnStyle =
		tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
			[diagonal].turnStyle;
		int curStart, w;

		curThing.units = curSpec.units - WelltrisInterface.MIN_UNITS +
				((curThing.getBonus()) ? 1 : 0);
		curStart = curThing.getRandomNumber() %
				tris[curThing.units].mode[diagonal].number
				[(curThing.getBonus()) ? WelltrisInterface.NOMIX :
				0][turnStyle];
		curThing.colorNumber = curStart;

		curThing.polyNumber = tris[curThing.units].mode
				[diagonal].start[curStart][turnStyle];
		curThing.size = tris[curThing.units].poly
				[curThing.polyNumber].size;
		curThing.position.x = generator.nextInt(diameter - curThing.size + 1);
		/* curThing.position.x = (diameter - curThing.size) / 2; */
		/* not all frozen else this is an infinite loop */
		if (!checkAllFrozen()) {
			do {
				w = generator.nextInt(WelltrisInterface.MAX_SIDES);
			} while (frozenWall[w] != 0);
			curThing.position.x += w * diameter;
			curThing.position.y = -tris[curThing.units].poly
				[curThing.polyNumber].startHeight[((suddenAppear) ? 1 : 0)];
		}
	}

	public void surface() {
		while (!atTop()) {
			curThing.position.y--;
		}
	}

	public void fall() {
		while (!atBottom()) {
			curThing.position.y++;
		}
	}

	public boolean move(int x, int y) {
		curThing.position.x += x;
		curThing.position.y += y;
		if (overlapping()) {
			curThing.position.x -= x;
			curThing.position.y -= y;
			return false;
		}
		return true;
	}

	public boolean rotate() {
		Thing oldThing = new Thing();
		oldThing.clone(curThing);
		rotateThing(true); // cw
		if (overlapping()) {
			curThing.clone(oldThing);
			return false;
		}
		return true;
	}

	public boolean reflect() {
		Thing oldThing = new Thing();
		oldThing.clone(curThing);
		reflectThing();
		if (overlapping()) {
			curThing.clone(oldThing);
			return false;
		}
		return true;
	}

	private void rotateThing(boolean cw) {
		if (tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
				[(curSpec.diagonal) ? 1 : 0].turnStyle > WelltrisInterface.NONE) {
			if (cw)
				for (int i = 0; i < WelltrisInterface.MAX_SIDES - 1; i++)
					curThing.polyNumber = tris[curThing.units].poly
						[curThing.polyNumber].rotation;
			else /* ccw */
				curThing.polyNumber = tris[curThing.units].poly
					[curThing.polyNumber].rotation;
		}
	}

	private void reflectThing() {
		/* reflect on y axis */
		if (tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
			[(curSpec.diagonal) ? 1 : 0].turnStyle == WelltrisInterface.ALL) {
			curThing.polyNumber = tris[curThing.units].poly
				[curThing.polyNumber].reflection;
		}
	}

	private void update() {
		// move thing
		/*if (x >= gameView.getWidth() - width - xSpeed || x + xSpeed <= 0) {
		   xSpeed = -xSpeed;
		}
		x = x + xSpeed;
		if (y >= gameView.getHeight() - height - ySpeed || y + ySpeed <= 0) {
		   ySpeed = -ySpeed;
		}
		y = y + ySpeed;
		currentFrame = ++currentFrame % BMP_COLUMNS;*/
	}

	public void onDraw(Canvas canvas) {
		update();
		/*int srcX = currentFrame * width;
		int srcY = getAnimationRow() * height;
		Rect src = new Rect(srcX, srcY, srcX + width, srcY + height);
		Rect dst = new Rect(x, y, x + width, y + height);
		canvas.drawBitmap(bmp, src, dst, null);*/
	}

	/*private int getAnimationRow() {
		double dirDouble = (Math.atan2(xSpeed, ySpeed) / (Math.PI / 2) + 2);
		int direction = (int) Math.round(dirDouble) % BMP_ROWS;
		return DIRECTION_TO_ANIMATION_MAP[direction];
	}*/

	/*public boolean isCollision(float x2, float y2) {
		return x2 > x && x2 < x + width
			&& y2 > y && y2 < y + height;
	}*/

	void drawBorder(Canvas canvas) { // no analog
		PolyominoImage.drawBorder(canvas, fieldOffset.x, fieldOffset.y,
			fieldOffset.x + 3 * WelltrisInterface.DEFAULT_WIDTH * boxWidth,
			fieldOffset.y + 3 * WelltrisInterface.DEFAULT_WIDTH * boxHeight);
	}


	void xorBaseUnit(Canvas canvas, PolyominoImage polyominoImage, int i, int j) {
			int color = PolyominoImage.colorThings[base[j][i].cid].getColor();
			{ // Simulate Xor
				int mask = 0xFFFFFF & color;
				color = color - mask + 0xFFFFFFFF - mask;
			}
			drawBaseUnit(canvas, polyominoImage, base[j][i].pmid,
				new Colour(color), i, j);
	}

	/*void drawField(Canvas canvas, PolyominoImage polyominoImage,
			int start, int finish) {
		int i, j;

		for (i = 0; i < perimeter; i++)
			for (j = start; j < finish; j++) {
				if (wall[j][i].pmid >= 0)
					polyominoImage.drawUnit(canvas, false,
						wall[j][i].pmid,
						wall[j][i].cid,
						fieldOffset.x, fieldOffset.y,
						i, j, boxWidth, boxHeight);
				else
					clearUnit(canvas, i, j);
			}
	}*/

	void drawField(Canvas canvas, PolyominoImage polyominoImage) {
		for (int j = 0; j < depth; j++) {
			for (int i = 0; i < perimeter; i++) {
				int color = 0xFF000000;
				if (wall[j][i].pmid >= 0) {
					color = PolyominoImage.colorThings[wall[j][i].cid].getColor();
					drawTrapazoid(canvas, polyominoImage,
						wall[j][i].pmid,
						color,
						(grid) ? Colour.black : background,
						i, j, PolyominoImage.DRAW_COLOR);
				}

				if (frozenWall[i / diameter] != 0) { // Simulate Xor
					Colour invert = new Colour(color);
					drawTrapazoid(canvas, polyominoImage,
						wall[j][i].pmid,
						invert.xorColor(),
						(grid) ? Colour.black : background,
						i, j,
						((wall[j][i].pmid >= 0) ?
						PolyominoImage.XOR_COLOR : PolyominoImage.XOR_CLEAR_COLOR));
				}
			}
		}
		for (int i = 0; i < diameter; i++)
			for (int j = 0; j < diameter; j++)
				if (base[j][i].pmid >= 0) {
				/*	System.out.println(i + " " + j + " "
				+ base[j][i].pmid + " " + base[j][i].cid
				+ fieldOffset.x + " " + fieldOffset.y);*/
					drawBaseUnit(canvas, polyominoImage,
						base[j][i].pmid, base[j][i].cid,
						i, j);
				}
	}

	void drawThing(Canvas canvas, PolyominoImage polyominoImage) {
		int i, j, xi, yj;
		int[][] tempBase = new int[diameter][diameter];

		/* Initialize */
		for (i = 0; i < diameter; i++)
			for (j = 0; j < diameter; j++)
				tempBase[j][i] = WelltrisInterface.EMPTY;
		/* Find pieces and overlap them if necessary */
		int[][] shape = tris[curThing.units].poly[curThing.polyNumber].shape;
		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++)
				if (shape[j][i] != 0) {
				  xi = (curThing.position.x + i + perimeter) % perimeter;
				  yj = curThing.position.y + j;
				  if (yj >= depth) {
					int rotPmid = rotateIndex(
					  PolyominoImage.polyPixmap(i, j,
					  shape,
					  curThing.size),
					  xi, yj);

					Point baseCoord = wallToBase(xi, yj);
					xi = checkWall(rotPmid, baseCoord.x, baseCoord.y);
					tempBase[baseCoord.y][baseCoord.x] &= xi;
				  }
				}
		/* Now draw */
		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++)
				if (shape[j][i] != 0) {
				  xi = (curThing.position.x + i + perimeter) % perimeter;
				  yj = curThing.position.y + j;
				  if (yj >= depth) {
					Point baseCoord = wallToBase(xi, yj);
					if (tempBase[baseCoord.y][baseCoord.x] != WelltrisInterface.EMPTY) {
					  /* drawBaseUnit(canvas, polyominoImage.polyPixmap(i, j,
					curThing, tempBase),
					curThing.colorNumber,
					baseCoord.x, baseCoord.y); */
					  /* This uses precomputed values */

					  drawBaseUnit(canvas, polyominoImage,
							  tempBase[baseCoord.y][baseCoord.x],
							  curThing.colorNumber,
							  baseCoord.x, baseCoord.y);
					  tempBase[baseCoord.y][baseCoord.x] = WelltrisInterface.EMPTY;
					}
				  } else if (yj >= 0) {
					int rotPmid = rotateIndex(
					  PolyominoImage.polyPixmap(i, j,
					  tris[curThing.units].poly[curThing.polyNumber].shape,
					  curThing.size),
					  xi, yj);

					drawTrapazoid(canvas, polyominoImage,
					  checkWallSides(checkWallBottom(rotPmid, yj), xi),
					  PolyominoImage.colorThings[curThing.colorNumber].getColor(),
					  (grid) ? Colour.black : background,
					  xi, yj, PolyominoImage.DRAW_COLOR);
				  }
				}
	}

	void drawThingDiff(Canvas canvas, PolyominoImage polyominoImage, Thing oldThing) {

				  /*polyominoImage.drawUnit(canvas, false,
						PolyominoImage.polyPixmap(i, j,
					tris[curThing.units].poly[curThing.polyNumber].shape,
					curThing.size),
					curThing.colorNumber,
					fieldOffset.x, fieldOffset.y,
					xi, yj, boxWidth, boxHeight);*/
				/* This uses precomputed values */
				/*  polyominoImage.drawUnit(canvas, false,
					PolyominoImage.NUM_BITMAPS - tris[curThing.units].poly
					  [curThing.polyNumber].shape[j][i] - 1,
					  curThing.colorNumber,
					  fieldOffset.x, fieldOffset.y,
					  xi, yj, boxWidth, boxHeight); */

					//clearUnit(canvas, xi, yj);
		int i, j, xi, yj, ox, oy;
		int[][] tempBase = new int[diameter][diameter];

		/* Initialize */
		for (i = 0; i < diameter; i++)
			for (j = 0; j < diameter; j++)
				tempBase[j][i] = WelltrisInterface.EMPTY;
		/* Find pieces and overlap them if necessary */
		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++)
				if (tris[curThing.units].poly
					  [curThing.polyNumber].shape[j][i] != 0) {
				  xi = (curThing.position.x + i + perimeter) % perimeter;
				  yj = curThing.position.y + j;
				  if (yj >= depth) {
					int rotPmid = rotateIndex(
					  PolyominoImage.polyPixmap(i, j,
					  tris[curThing.units].poly[curThing.polyNumber].shape,
					  curThing.size),
					  xi, yj);

					Point baseCoord = wallToBase(xi, yj);
					xi = checkWall(rotPmid, baseCoord.x, baseCoord.y);
					tempBase[baseCoord.y][baseCoord.x] &= xi;
				  }
				}
		/* Test to make corner split neighbor awareness */
		for (i = 0; i < diameter; i++)
			for (j = 0; j < diameter; j++) {
			  if (tempBase[j][i] >= 0) {
				if ((j > 0) && (tempBase[j - 1][i] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.UP;
				}
				if ((j < diameter - 1) &&
				(tempBase[j + 1][i] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.DOWN;
				}
				if (i > 0 && (tempBase[j][i - 1] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.LEFT;
				}
				if ((i < diameter - 1) &&
				(tempBase[j][i + 1] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.RIGHT;
				}
				if (i > 0 && j > 0 && (tempBase[j - 1][i - 1] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.LEFT_UP;
				}
				if (i > 0 && (j < diameter - 1) &&
				(tempBase[j + 1][i - 1] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.LEFT_DOWN;
				}
				if ((i < diameter - 1) && j > 0 &&
				(tempBase[j - 1][i + 1] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.RIGHT_UP;
				}
				if ((i < diameter - 1) && (j < diameter - 1) &&
				(tempBase[j + 1][i + 1] != WelltrisInterface.EMPTY)) {
				  tempBase[j][i] &= ~PolyominoImage.RIGHT_DOWN;
				}
			  }
			}
		/* Now draw */
		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++)
				if (tris[curThing.units].poly
					  [curThing.polyNumber].shape[j][i] != 0) {
				  xi = (curThing.position.x + i + perimeter) % perimeter;
				  yj = curThing.position.y + j;
				  if (yj >= depth) {
					Point baseCoord = wallToBase(xi, yj);
					if (tempBase[baseCoord.y][baseCoord.x] >= 0) {
					  /* drawBaseUnit(canvas, polyominoImage.polyPixmap(baseCoord.x, baseCoord.y,
					curThing, tempBase),
					curThing.colorNumber,
					baseCoord.x, baseCoord.y); */
					/* This uses precomputed values */
					  drawBaseUnit(canvas, polyominoImage,
							  tempBase[baseCoord.y][baseCoord.x],
							  curThing.colorNumber,
							  baseCoord.x, baseCoord.y);
					  /* negate for erase */
					  tempBase[baseCoord.y][baseCoord.x] =
							  -tempBase[baseCoord.y][baseCoord.x] - 2;
					}
				  } else if (yj >= 0) {
					int rotPmid = rotateIndex(
						PolyominoImage.polyPixmap(i, j,
						tris[curThing.units].poly[curThing.polyNumber].shape,
						curThing.size),
						xi, yj);

					drawTrapazoid(canvas, polyominoImage,
					  checkWallSides(checkWallBottom(rotPmid, yj), xi),
					  PolyominoImage.colorThings[curThing.colorNumber].getColor(),
					  (grid) ? Colour.black : background,
					  xi, yj, PolyominoImage.DRAW_COLOR);
				  }
				}
		/* Erase */
		for (i = 0; i < oldThing.size; i++)
			for (j = 0; j < oldThing.size; j++) {
				xi = (oldThing.position.x + i + perimeter) % perimeter;
				yj = oldThing.position.y + j;
				ox = (xi - curThing.position.x + perimeter) % perimeter;
				oy = yj - curThing.position.y;
				if (yj >= depth) {
				  Point baseCoord = wallToBase(xi, yj);
				  if (tris[oldThing.units].poly
					  [oldThing.polyNumber].shape[j][i] != 0 &&
					  ((ox >= curThing.size) || (oy < 0) ||
					  (oy >= curThing.size) ||
					  tempBase[baseCoord.y][baseCoord.x] == WelltrisInterface.EMPTY)) {
					clearUnit(canvas, baseCoord.x, baseCoord.y);
				  }
				} else if (yj >= 0 &&
					  (tris[oldThing.units].poly
					  [oldThing.polyNumber].shape[j][i] != 0) &&
					  ((ox >= curThing.size) || (oy < 0) ||
					  (oy >= curThing.size) || tris[curThing.units].poly
					  [curThing.polyNumber].shape[oy][ox] == 0)) {
					drawTrapazoid(canvas, polyominoImage,
					  PolyominoImage.NUM_BITMAPS - 1,
					  background,
					  (grid) ? Colour.black : background,
					  xi, yj, PolyominoImage.CLEAR_COLOR);
				}
			}
	}

	void drawUnit(Canvas canvas, PolyominoImage polyominoImage,
			int pmid, int cid, int i, int j) {
		if (j >= 0)
                        return;
                if (j < depth) {
                        drawTrapazoid(canvas, polyominoImage, pmid,
                                PolyominoImage.colorThings[cid].getColor(),
                                ((grid) ? Colour.black : background),
                                i, j, PolyominoImage.DRAW_COLOR);
                } else {
                        Point baseCoord = wallToBase(i, j);
                        drawBaseUnit(canvas, polyominoImage, pmid, cid,
                                baseCoord.x, baseCoord.y);
		}
	}

	void putUnit() {
		int i, j, xi, yj;
		int[][] tempBase = new int[diameter][diameter];

		/* Initialize */
		for (i = 0; i < diameter; i++)
			for (j = 0; j < diameter; j++)
				tempBase[j][i] = WelltrisInterface.EMPTY;
		/* Set pieces and overlap them if necessary */
		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++)
				if (tris[curThing.units].poly
				      [curThing.polyNumber].shape[j][i] != 0) {
				  xi = (curThing.position.x + i + perimeter) % perimeter;
				  yj = curThing.position.y + j;
				  if (yj >= depth) {
				    int rotPmid = rotateIndex(PolyominoImage.polyPixmap(i, j,
				      tris[curThing.units].poly[curThing.polyNumber].shape,
				      curThing.size), xi, yj);

				    Point baseCoord = wallToBase(xi, yj);
				    xi = checkWall(rotPmid, baseCoord.x, baseCoord.y);
				    tempBase[baseCoord.y][baseCoord.x] &= xi;
				  } else if (yj >= 0) {
				    int rotPmid = rotateIndex(PolyominoImage.polyPixmap(i, j,
				      tris[curThing.units].poly[curThing.polyNumber].shape,
				      curThing.size), xi, yj);

				    wall[yj][xi].pmid =
				      checkWallSides(checkWallBottom(rotPmid, yj), xi);
				    wall[yj][xi].cid = curThing.colorNumber;
				    frozenWall[xi / diameter] = WelltrisInterface.MAX_SIDES;
				  }
				}
		/* Test to make corner split neighbor awareness */
		for (i = 0; i < diameter; i++)
			for (j = 0; j < diameter; j++) {
			  if (tempBase[j][i] >= 0) {
			    if (j > 0 && (tempBase[j - 1][i] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.UP;
			    }
			    if ((j < diameter - 1) &&
					(tempBase[j + 1][i] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.DOWN;
			    }
			    if (i > 0 && (tempBase[j][i - 1] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.LEFT;
			    }
			    if ((i < diameter - 1) &&
					(tempBase[j][i + 1] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.RIGHT;
			    }
			    if (i > 0 && j > 0 &&
					(tempBase[j - 1][i - 1] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.LEFT_UP;
			    }
			    if (i > 0 && (j < diameter - 1) &&
					(tempBase[j + 1][i - 1] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.LEFT_DOWN;
			    }
			    if ((i < diameter - 1) && j > 0 &&
					(tempBase[j - 1][i + 1] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.RIGHT_UP;
			    }
			    if ((i < diameter - 1) && (j < diameter - 1) &&
					(tempBase[j + 1][i + 1] != WelltrisInterface.EMPTY)) {
				tempBase[j][i] &= ~PolyominoImage.RIGHT_DOWN;
			    }
			  }
			}
		for (i = 0; i < diameter; i++)
			for (j = 0; j < diameter; j++)
				if (tempBase[j][i] != WelltrisInterface.EMPTY) {
					base[j][i].pmid = tempBase[j][i];
					base[j][i].cid = curThing.colorNumber;
				}
	}

	boolean overlapping() {
		int i, j, xi, yj;
		int x = curThing.position.x, y = curThing.position.y;

		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++)
			  if (tris[curThing.units].poly
				  [curThing.polyNumber].shape[j][i] != 0) {
				xi = (x + i + perimeter) % perimeter;
				yj = y + j;
				if (yj < depth) {
				  if (frozenWall[xi / diameter] != 0)
					return true;
				  if (suddenAppear) { /* creationalism */
/* This method does not allow turning polyomino to an area above screen */
				    if ((yj < 0) || (wall[yj][xi].pmid >= 0))
				      return true;
				  } else { /* evolution */
/* This method one can turn polyomino to an area above of screen, also
   part of the polyomino may not be visible initially */
				    if ((yj >= 0) && (wall[yj][xi].pmid >= 0))
				      return true;
				  }
				} else if (yj < depth + diameter) {
				  Point baseCoord = wallToBase(xi, yj);
				  if (base[baseCoord.y][baseCoord.x].pmid >= 0)
				    return true;
				} else
				  return true;
			  }
		return false;
	}

	Point wallToBase(int wall_x, int wall_y) {
		switch (wall_x / diameter) {
		case 0:
			return new Point(wall_x, wall_y - depth);
		case 1:
			return new Point (diameter - 1 + depth - wall_y,
				wall_x - diameter);
		case 2:
			return new Point(perimeter - 1 - diameter - wall_x,
				diameter - 1 + depth - wall_y);
		case 3:
			return new Point(wall_y - depth,
				perimeter - 1 - wall_x);
		default:
			System.err.println("wallToBase kinds range 0-3, got " +
				wall_x / diameter);
			return new Point(-1, -1);
		}
	}

	int checkWall(int pmid, int baseI, int baseJ) {
		int newPmid = pmid;

		if (newPmid < 0)
			return newPmid;
		/* Check if next to well wall */
		if (baseI == 0)
			newPmid = PolyominoImage.CHECKLEFT(newPmid);
		if (baseJ == 0)
			newPmid = PolyominoImage.CHECKUP(newPmid);
		if (baseI == diameter - 1)
			newPmid = PolyominoImage.CHECKRIGHT(newPmid);
		if (baseJ == diameter - 1)
			newPmid = PolyominoImage.CHECKDOWN(newPmid);
		return newPmid;
	}

	int checkSides(int pmid, int i) {
		int newPmid = pmid;

		if (newPmid < 0)
			return newPmid;
		if (i == 0 || i == 3 * diameter - 1)
			newPmid = PolyominoImage.CHECKLEFT(newPmid);
		else if (i == 2 * diameter || i == diameter - 1)
			newPmid = PolyominoImage.CHECKRIGHT(newPmid);
		else if (i ==  diameter || i == 4 * diameter - 1)
			newPmid = PolyominoImage.CHECKUP(newPmid);
		else if (i == 3 * diameter || i == 2 * diameter - 1)
			newPmid = PolyominoImage.CHECKDOWN(newPmid);
		return newPmid;
	}

	int checkBottom(int pmid, int i, int j) {
		if (pmid < 0 || j != depth - 1)
			return pmid;
		switch (i / diameter) {
		case 0:
			return PolyominoImage.CHECKDOWN(pmid);
		case 1:
			return PolyominoImage.CHECKLEFT(pmid);
		case 2:
			return PolyominoImage.CHECKUP(pmid);
		case 3:
			return PolyominoImage.CHECKRIGHT(pmid);
		default:
		}
		return WelltrisInterface.EMPTY;
	}

	int checkWallSides(int pmid, int i) {
		int newPmid = pmid;

		if (newPmid < 0)
			return newPmid;
		if (i % diameter == 0)
			newPmid = PolyominoImage.CHECKLEFT(newPmid);
		else if (i % diameter == diameter - 1)
			newPmid = PolyominoImage.CHECKRIGHT(newPmid);
		return newPmid;
	}

	int checkWallBottom(int pmid, int j) {
		if (pmid < 0 || j != depth - 1)
			return pmid;
		return PolyominoImage.CHECKDOWN(pmid);
	}

	int rotateIndex(int pmid, int i, int j) {
		int wallRotation, pm1, pm0;

		if (pmid == WelltrisInterface.EMPTY)
			return WelltrisInterface.EMPTY;
		if (j < depth)
			return pmid;
		wallRotation = (WelltrisInterface.MAX_SIDES - i /
			diameter) % WelltrisInterface.MAX_SIDES;
		pm1 = (0xF0 & pmid) << wallRotation;
		pm0 = (0xF & pmid) << wallRotation;
		pm1 = (0xF0 & pm1) | ((pm1 & 0xF00) >> WelltrisInterface.MAX_SIDES);
		pm0 = (0xF & pm0) | ((pm0 & 0xF0) >> WelltrisInterface.MAX_SIDES);
		return (pm1 | pm0);
	}

	boolean atTop() {
		int i, j, xi, yj;
		int x = curThing.position.x, y = curThing.position.y;

		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++) {
			  xi = x + i;
			  yj = y + j;
			  if ((yj >= -1) && tris[curThing.units].poly
				  [curThing.polyNumber].shape[j][i] != 0 &&
				  ((yj <= 0) ||
				   (xi < 0) || (xi >= perimeter) ||
				   (wall != null && wall[yj - 1][xi].pmid >= 0)))
				return true;
			}
		return false;
	}

	boolean atBottom() {
		int i, j, xi, yj;
		int x = curThing.position.x, y = curThing.position.y;

		for (i = 0; i < curThing.size; i++)
			for (j = 0; j < curThing.size; j++)
			  if (tris[curThing.units].poly
				  [curThing.polyNumber].shape[j][i] != 0) {
				xi = (x + i) % perimeter;
				yj = y + j;
				if (yj < -1)
				  return false;
				if (yj < depth - 1) {
				  if (frozenWall[xi / diameter] != 0)
				return true;
				  if (wall[yj + 1][xi].pmid >= 0)
				return true;
				} else if (yj < depth + diameter - 1) {
				  // int baseCoord.x, baseCoord.y;
				  Point baseCoord = wallToBase(xi, yj + 1);
				  if (base[baseCoord.y][baseCoord.x].pmid >= 0)
				return true;
				} else
				  return true;
			  }
		return false;
	}

	boolean atBaseFully() {
		for (int i = 0; i < curThing.size; i++)
			for (int j = 0; j < curThing.size; j++)
			  if (curThing.position.y + j < depth && tris[curThing.units].poly
				  [curThing.polyNumber].shape[j][i] != 0)
				 return false;
		return true;
	}

	boolean atBasePartially() {
		for (int i = 0; i < curThing.size; i++)
			for (int j = 0; j < curThing.size; j++)
			  if (curThing.position.y + j >= depth && tris[curThing.units].poly
				  [curThing.polyNumber].shape[j][i] != 0)
				 return true;
		return false;
	}

	boolean wallChange(Thing oldthing, Thing newthing) {
		int		 w = -1, i, j;
		int		 x = curThing.position.x;

		for (i = 0; i < oldthing.size; i++)
			for (j = 0; j < oldthing.size; j++)
			  if (tris[oldthing.units].poly[oldthing.polyNumber].shape[j][i] != 0) {
				if (w == -1)
				  w = ((x + i) % perimeter) / diameter;
				else if (w != ((x + i) % perimeter) / diameter)
				  return true;
			  }
		for (i = 0; i < newthing.size; i++)
			for (j = 0; j < newthing.size; j++)
			  if (tris[newthing.units].poly[newthing.polyNumber].shape[j][i] != 0 &&
				  (w != ((x + i) % perimeter) / diameter))
				return true;
		return false;
	}

	void initFill(boolean fixedFill, int fill) {
		int i, j, col, fixedUnits;
		int turnStyle = tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
			[(curSpec.diagonal) ? 1 : 0].turnStyle;
		int numberPolyominoes = tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
			[(curSpec.diagonal) ? 1 : 0].number[(curSpec.mixed) ? 1 : 0][turnStyle];

		if (fixedFill) {
			/* This algorithm randomly picks fixedUnits per row
			 * (4 for DEFAULTDIAMETER, ie. 8/2)
			 * This is more in keeping with the original */
			fixedUnits = diameter / 2;
			for (j = 0; j < depth; j++) {
				for (i = 0; i < perimeter; i++) {
					wall[j][i].pmid = -1;
					wall[j][i].cid = 0;
				}
				if (j >= depth - fill)
					for (i = 0; i < fixedUnits; i++) {
						do
							col = generator.nextInt(perimeter);
						while (wall[j][col].pmid != -1);
						wall[j][col].pmid = PolyominoImage.NUM_BITMAPS - 1;
						wall[j][col].cid = generator.nextInt(numberPolyominoes);
					}
			}
		} else {
			/* This algorithm randomly fills boxes to a certain level
			   The problem with this is that it could potentially fill
			   up a whole row */
			for (j = 0; j < depth; j++)
				for (i = 0; i < perimeter; i++)
					if ((j >= depth - fill) &&
							(generator.nextInt(8) == 0)) {
						wall[j][i].pmid = PolyominoImage.NUM_BITMAPS - 1;
						wall[j][i].cid = generator.nextInt(numberPolyominoes);
					} else {
						wall[j][i].pmid = -1;
						wall[j][i].cid = 0;
					}
		}
	}

	void fillLines(Canvas canvas, PolyominoImage polyominoImage, int j) {
		int turnStyle = tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
			[(curSpec.diagonal) ? 1 : 0].turnStyle;
		int pmid = PolyominoImage.NUM_BITMAPS - 1;

		int dir = generator.nextInt(2);  // spin direction

		for (int i = 0; i < perimeter; i++) {
				int cid = generator.nextInt(tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
						[(curSpec.diagonal) ? 1 : 0].number[(curSpec.mixed) ? 1 : 0]
						[turnStyle]);
			int k = i;

			if (dir == 1)
				k = perimeter - 1 - i;
			wall[j][k].pmid = pmid;
			wall[j][k].cid = cid;
			if (j > depth) {
				Point baseCoord = wallToBase(k, j);
				base[baseCoord.y][baseCoord.x].pmid = pmid;
				base[baseCoord.y][baseCoord.x].cid = cid;
			}
			drawUnit(canvas, polyominoImage,
				pmid, cid,
				k, j);
			cid = generator.nextInt(tris[curSpec.units - WelltrisInterface.MIN_UNITS].mode
				[(curSpec.diagonal) ? 1 : 0].number[(curSpec.mixed) ? 1 : 0]
				[turnStyle]);
			wall[depth + diameter - j - 1][k].pmid = pmid;
			wall[depth + diameter - j - 1][k].cid = cid;
			if ( diameter - j - 1 > 0) {
				Point baseCoord = wallToBase(k, depth + diameter - j - 1);
				base[baseCoord.y][baseCoord.x].pmid = pmid;
				base[baseCoord.y][baseCoord.x].cid = cid;
			}
			drawUnit(canvas, polyominoImage,
				pmid, cid,
				k, depth + diameter - j - 1);
		}
	}

	void freezeWall(Canvas canvas, PolyominoImage polyominoImage, int w) {
		System.out.println("DAB3 freezing wall");
		for (int j = 0; j < depth; j++)
			for (int i = 0; i < diameter; i++) {
				int color;
				if (wall[j][w * diameter + i].pmid < 0)
					color = background;
				else
					color = PolyominoImage.colorThings[wall[j][w * diameter + i].cid].getColor();
				if (frozenWall[w] == 0) {
					drawTrapazoid(canvas, polyominoImage,
						wall[j][w * diameter + i].pmid,
						color,
						(grid) ? Colour.black : background,
						w * diameter + i, j,
						((wall[j][w * diameter + i].pmid < 0) ?
								PolyominoImage.CLEAR_COLOR : PolyominoImage.DRAW_COLOR));
				} else { // Simulate Xor
					//color = (new Colour(color)).xorColor();
					drawTrapazoid(canvas, polyominoImage,
						wall[j][w * diameter + i].pmid,
						//color,
						(new Colour(color)).xorColor(),
						(grid) ? Colour.black : background,
						w * diameter + i, j,
						((wall[j][w * diameter + i].pmid < 0) ?
								PolyominoImage.CLEAR_COLOR : PolyominoImage.DRAW_COLOR));
						//PolyominoImage.XOR_CLEAR_COLOR : PolyominoImage.XOR_COLOR));
				}
			}
		/*if (((WelltrisApplet)applet).getToggleSound())
			((WelltrisApplet)applet).playDripAudio();*/
	}

	final void unfreezeWalls() {
		int i;

		for (i = 0; i < WelltrisInterface.MAX_SIDES; i++)
			frozenWall[i] = 0;
	}

	void dropWall(Canvas canvas, PolyominoImage polyominoImage, int w) {
		int i, j, k, l, lines = 0;
		boolean fits;

		for (j = depth - 1; j >= 0 && lines == 0; j--)
			for (i = 0; i < diameter; i++)
				if (wall[j][w * diameter + i].pmid != WelltrisInterface.EMPTY)
					lines = depth - j;
		if (lines > 0) {
			fits = true;
			j = 0;
			while (j < (diameter / 2 + lines - 1) && fits) {
				j++;
				for (l = depth - j; l < depth; l++)
				  for (i = 0; i < diameter; i++)
				    if (wall[l][w * diameter + i].pmid != WelltrisInterface.EMPTY) {
				      Point baseCoord = wallToBase(w * diameter + i, l + j);
				      if (base[baseCoord.y][baseCoord.x].pmid != WelltrisInterface.EMPTY)
					fits = false;
				    }
			}
			if (!fits)
				j--;
			if (j > 0) {
				for (l = depth - 1; l >= 0; l--)
				  for (i = 0; i < diameter; i++)
				    if (wall[l][w * diameter + i].pmid != WelltrisInterface.EMPTY) {
				      k = w * diameter + i;
				      if (l + j >= depth) {
					Point baseCoord = wallToBase(k, l + j);
					base[baseCoord.y][baseCoord.x].cid = wall[l][k].cid;
					base[baseCoord.y][baseCoord.x].pmid = checkWall(rotateIndex(wall[l][k].pmid,
					  k, l + j), baseCoord.x, baseCoord.y);
				       wall[l][k].pmid = WelltrisInterface.EMPTY;
				       drawTrapazoid(canvas, polyominoImage,
						   PolyominoImage.NUM_BITMAPS - 1,
						   background,
						   (grid) ? Colour.black : background,
						   k, l, PolyominoImage.CLEAR_COLOR);
				       drawBaseUnit(canvas, polyominoImage,
						   base[baseCoord.y][baseCoord.x].pmid, base[baseCoord.y][baseCoord.x].cid,
						   baseCoord.x, baseCoord.y);
				     } else {
				       wall[l + j][k].cid = wall[l][k].cid;
				       wall[l + j][k].pmid = checkWallBottom(wall[l][k].pmid, l + j);
				       wall[l][k].pmid = WelltrisInterface.EMPTY;
				       drawTrapazoid(canvas, polyominoImage,
						  PolyominoImage.NUM_BITMAPS - 1,
						  background,
						  (grid) ? Colour.black : background,
						  k, l, PolyominoImage.CLEAR_COLOR);
				       drawTrapazoid(canvas, polyominoImage, wall[l + j][k].pmid,
						  PolyominoImage.colorThings[wall[l][k].cid].getColor(),
						  (grid) ? Colour.black : background,
						  k, l + j, PolyominoImage.DRAW_COLOR);
				    }
				  }
			}
		}
		/*if (((WelltrisApplet)applet).getToggleSound())
			((WelltrisApplet)applet).playDripAudio();*/
	}

	boolean allFrozen(Canvas canvas, PolyominoImage polyominoImage) {
		for (int w = 0; w < WelltrisInterface.MAX_SIDES; w++)
			if (frozenWall[w] == WelltrisInterface.MAX_SIDES) {
				freezeWall(canvas, polyominoImage, w);
			}
		return checkAllFrozen();
	}

	boolean checkAllFrozen() {
		for (int w = 0; w < WelltrisInterface.MAX_SIDES; w++)
			if ((frozenWall[w] == 0))
				return false;
		return true;
	}

	boolean checkFreeze(Canvas canvas, PolyominoImage polyominoImage) {
		boolean freeze = false;
		int w;

		if (generator.nextInt(2) == 1) {
			for (w = 0; w < WelltrisInterface.MAX_SIDES; w++) {
				if ((frozenWall[w] == 0))
					dropWall(canvas, polyominoImage, w);
				else {
					frozenWall[w]--;
					if ((frozenWall[w] == 0)) {
						freezeWall(canvas, polyominoImage, w);
						dropWall(canvas, polyominoImage, w);
						freeze = true;
					}
				}
			}
		} else {
			for (w = WelltrisInterface.MAX_SIDES - 1; w >= 0; w--) {
				if ((frozenWall[w] == 0))
					dropWall(canvas, polyominoImage, w);
				else {
					frozenWall[w]--;
					if ((frozenWall[w] == 0)) {
						freezeWall(canvas, polyominoImage, w);
						dropWall(canvas, polyominoImage, w);
						freeze = true;
					}
				}
			}
		}
		return freeze;
	}

	int checkLines() {
		Field[][] tempBase = new Field[diameter][diameter];
		int[][] lSet = new int[2][diameter];
		int[] nSet = new int[2];
		int i, j;

		nSet[0] = nSet[1] = 0;

		for (j = 0; j < diameter; j++)
			for (i = 0; i < diameter; i++) {
				tempBase[j][i] = new Field();
				tempBase[j][i].pmid = base[j][i].pmid;
				tempBase[j][i].cid = base[j][i].cid;
			}
		for (j = 0; j < diameter; j++) {
			/* Find filled depth */
			lSet[0][j] = 0;
			for (i = 0; i < diameter; i++)
				if (tempBase[j][i].pmid >= 0)
					lSet[0][j]++;
			if (lSet[0][j] == diameter)
				nSet[0]++;
			/* Find filled perimeter */
			lSet[1][j] = 0;
			for (i = 0; i < diameter; i++)
				if (tempBase[i][j].pmid >= 0)
					lSet[1][j]++;
			if (lSet[1][j] == diameter)
				nSet[1]++;
		}
		return (nSet[0] + nSet[1]);
	}

	int invertLines(Canvas canvas, PolyominoImage polyominoImage) {
		Field[][] tempBase = new Field[diameter][diameter];
		int[][] lSet = new int[2][diameter];
		int[] nSet = new int[2];
		int i, j;

		nSet[0] = nSet[1] = 0;

		for (j = 0; j < diameter; j++)
			for (i = 0; i < diameter; i++) {
				tempBase[j][i] = new Field();
				tempBase[j][i].pmid = base[j][i].pmid;
				tempBase[j][i].cid = base[j][i].cid;
			}
		for (j = 0; j < diameter; j++) {
			/* Find filled depth */
			lSet[0][j] = 0;
			for (i = 0; i < diameter; i++)
				if (tempBase[j][i].pmid >= 0)
					lSet[0][j]++;
			if (lSet[0][j] == diameter)
				nSet[0]++;
			/* Find filled perimeter */
			lSet[1][j] = 0;
			for (i = 0; i < diameter; i++)
				if (tempBase[i][j].pmid >= 0)
					lSet[1][j]++;
			if (lSet[1][j] == diameter)
				nSet[1]++;
		}
		if ((nSet[0] != 0) || (nSet[1] != 0)) {

				for (j = 0; j < diameter; j++) {
					if (lSet[0][j] == diameter)
					  for (i = 0; i < diameter; i++) {
						  xorBaseUnit(canvas, polyominoImage, i, j);
					  }
					if (lSet[1][j] == diameter)
					  for (i = 0; i < diameter; i++) {
						  xorBaseUnit(canvas, polyominoImage, j, i);
					  }
				}
		}
		return (nSet[0] + nSet[1]);
	}

	int shrinkLines(Canvas canvas, PolyominoImage polyominoImage) {
		Field[][] tempBase = new Field[diameter][diameter];
		int[][] lSet = new int[2][diameter];
		int[] nSet = new int[2];
		int i, j, k;

		nSet[0] = nSet[1] = 0;

		for (j = 0; j < diameter; j++)
			for (i = 0; i < diameter; i++) {
				tempBase[j][i] = new Field();
				tempBase[j][i].pmid = base[j][i].pmid;
				tempBase[j][i].cid = base[j][i].cid;
			}
		for (j = 0; j < diameter; j++) {
			/* Find filled depth */
			lSet[0][j] = 0;
			for (i = 0; i < diameter; i++)
				if (tempBase[j][i].pmid >= 0)
					lSet[0][j]++;
			if (lSet[0][j] == diameter)
				nSet[0]++;
			/* Find filled perimeter */
			lSet[1][j] = 0;
			for (i = 0; i < diameter; i++)
				if (tempBase[i][j].pmid >= 0)
					lSet[1][j]++;
			if (lSet[1][j] == diameter)
				nSet[1]++;
		}
			if (nSet[0] != 0) {
				for (j = 0; j < diameter / 2; j++)
					if (lSet[0][j] == diameter)
					  for (i = 0; i < diameter; i++) {
					    for (k = j; k > 0; k--) {
					      tempBase[k][i].pmid =
						tempBase[k - 1][i].pmid;
					      tempBase[k][i].cid =
						tempBase[k - 1][i].cid;
					    }
					    tempBase[0][i].pmid = WelltrisInterface.EMPTY;
					    if ((j > 0) &&
						(tempBase[j][i].pmid >= 0)) {
					      tempBase[j][i].pmid =
						PolyominoImage.CHECKDOWN(
						  tempBase[j][i].pmid);
					    }
					    if (tempBase[j + 1][i].pmid >= 0)
					      tempBase[j + 1][i].pmid =
						PolyominoImage.CHECKUP(
						  tempBase[j + 1][i].pmid);
					  }
				for (j = diameter - 1; j >= diameter / 2; j--)
					if (lSet[0][j] == diameter)
					  for (i = 0; i < diameter; i++) {
					    for (k = j; k < diameter - 1; k++) {
					      tempBase[k][i].pmid =
						tempBase[k + 1][i].pmid;
					      tempBase[k][i].cid =
						tempBase[k + 1][i].cid;
					    }
					    tempBase[diameter - 1][i].pmid = WelltrisInterface.EMPTY;
					    if ((j < diameter - 1) &&
						(tempBase[j][i].pmid >= 0)) {
					      tempBase[j][i].pmid =
						PolyominoImage.CHECKUP(
						 tempBase[j][i].pmid);
					    }
					    if (tempBase[j - 1][i].pmid >= 0)
					      tempBase[j - 1][i].pmid =
						PolyominoImage.CHECKDOWN(
						  tempBase[j - 1][i].pmid);
					  }
			}
			if (nSet[1] != 0) {
				for (i = 0; i < diameter / 2; i++)
					if (lSet[1][i] == diameter)
					  for (j = 0; j < diameter; j++) {
					    for (k = i; k > 0; k--) {
					      tempBase[j][k].pmid =
						tempBase[j][k - 1].pmid;
					      tempBase[j][k].cid =
						tempBase[j][k - 1].cid;
					    }
					    tempBase[j][0].pmid = WelltrisInterface.EMPTY;
					    if ((i > 0) &&
						(tempBase[j][i].pmid >= 0)) {
					      tempBase[j][i].pmid =
						PolyominoImage.CHECKRIGHT(
						  tempBase[j][i].pmid);
					    }
					    if (tempBase[j][i + 1].pmid >= 0)
					      tempBase[j][i + 1].pmid =
						PolyominoImage.CHECKLEFT(
						  tempBase[j][i + 1].pmid);
					  }
				for (i = diameter - 1; i >= diameter / 2; i--)
					if (lSet[1][i] == diameter)
					  for (j = 0; j < diameter; j++) {
					    for (k = i; k < diameter - 1; k++) {
					      tempBase[j][k].pmid =
						tempBase[j][k + 1].pmid;
					      tempBase[j][k].cid =
						tempBase[j][k + 1].cid;
					    }
					    tempBase[j][diameter - 1].pmid =
					      WelltrisInterface.EMPTY;
					    if ((i < diameter - 1) &&
						(tempBase[j][i].pmid >= 0)) {
					      tempBase[j][i].pmid =
						PolyominoImage.CHECKLEFT(
						  tempBase[j][i].pmid);
					    }
					    if (tempBase[j][i - 1].pmid >= 0)
					      tempBase[j][i - 1].pmid =
						PolyominoImage.CHECKRIGHT(
						  tempBase[j][i - 1].pmid);
					  }
			}
			for (j = 0; j < diameter; j++)
				for (i = 0; i < diameter; i++)
					if ((tempBase[j][i].cid != base[j][i].cid) ||
					    (tempBase[j][i].pmid != base[j][i].pmid)) {
					  base[j][i].pmid = tempBase[j][i].pmid;
					  base[j][i].cid = tempBase[j][i].cid;
					  if (base[j][i].pmid >= 0)
					    drawBaseUnit(canvas, polyominoImage, base[j][i].pmid, base[j][i].cid, i, j);
					  else
					    clearUnit(canvas, i, j);
					}
		/*if (((WelltrisApplet)applet).getToggleSound())
			((WelltrisApplet)applet).playBumpAudio();*/
		return (nSet[0] + nSet[1]);
	}

	void drawGrid(Canvas canvas) {
		int color = (grid) ? Colour.white : background;
		int invColor = (grid) ? background : Colour.white;

		for (int i = 0; i < perimeter; i++)
			for (int k = 0; k < depth; k++) {
				if (frozenWall[i / diameter] != 0) {
					drawTrapazoidBorder(canvas, invColor, i, k);
				} else {
					drawTrapazoidBorder(canvas, color, i, k);
				}
			}
		/* int i, k, square_x, sum, sum_var;
		sum = sum_var = diameter * boxSize + delta - 1;
		square_x = 0;
		for (i = 0; i < diameter; i++) {
			g.drawLine(square_x, 0, sum_var, sum);
			g.drawLine(endPt - 1, square_x, endPt - 1 - sum, sum_var);
			g.drawLine(endPt - 1 - square_x, endPt - 1,
				endPt - 1 - sum_var, endPt - 1 - sum);
			g.drawLine(0, endPt - 1 - square_x, sum, endPt - 1 - sum_var);
			square_x += 3 * boxSize;
			sum_var += boxSize;
		}
		sum = endPt;
		square_x = 0;
		k = (diameter * boxSize) / depth;
		for (i = 0; i <= depth; i++) {
			g.fillRect(square_x, square_x, 1, sum);
			g.fillRect(square_x, square_x, sum, 1);
			g.fillRect(square_x + sum - 1, square_x, 1, sum);
			g.fillRect(square_x, square_x + sum - 1, sum, 1);
			square_x += k;
			sum -= (2 * k);
		}*/
		/* sum = diameter * boxSize + delta;
		square_x = square_y = sum;
		for (i = 0; i <= diameter ; i++) {
			g.fillRect(square_x, square_y, 1, sum);
			square_x += boxSize;
		}
		square_x = square_y = sum;
		for (i = 0; i <= diameter ; i++) {
			g.fillRect(square_x, square_y, sum, 1);
			square_y += boxSize;
		} */
	}

	void drawTrapazoid(Canvas canvas, PolyominoImage polyominoImage,
			int pmid, int cid, int bid,
			int i, int j, int draw) {
		int ulx, uly, lrx, lry, trapBase, plateau, trapBaseWidth, trapEndPt, sideWall;
		int newI = i;

		trapBaseWidth =  diameter * boxSize + 1;
		trapEndPt = 3 * trapBaseWidth;
		sideWall = newI / diameter;
		newI -= (sideWall * diameter);
		ulx = (trapEndPt +  (2 * newI - diameter) * ((depth / 2 +
			2 * boxSize * (depth - j)) / depth + boxSize)) / 2;
		uly = (j * trapBaseWidth + depth / 2) / depth;
		lrx = (trapEndPt + (2  + 2 * newI  - diameter) * ((depth / 2 +
			2 * boxSize * (depth - j - 1)) / depth + boxSize)) / 2;
		lry = ((j + 1) * trapBaseWidth + depth / 2) / depth;
		trapBase = boxSize + ((depth - j) * 2 * boxSize + depth / 2) /
			depth;
		plateau = boxSize + ((depth - j - 1) * 2 * boxSize +
			depth / 2) / depth;

		switch (sideWall) {
		case 0:
			polyominoImage.drawWallUnit(canvas, pmid, cid, bid,
				fieldOffset.x, fieldOffset.y,
				ulx + 1, uly, trapBase, 0,
				lrx + 1, lry, plateau, 0,
				draw);
			break;
		case 1:
			polyominoImage.drawWallUnit(canvas, pmid, cid, bid,
				fieldOffset.x, fieldOffset.y,
				trapEndPt - uly, ulx + 1, 0, trapBase,
				trapEndPt - lry, lrx + 1, 0, plateau,
				draw);
			break;
		case 2:
			polyominoImage.drawWallUnit(canvas,
				pmid + PolyominoImage.NUM_BITMAPS,
				cid, bid,
				fieldOffset.x, fieldOffset.y,
				trapEndPt - 1 - ulx, trapEndPt - uly, -trapBase, 0,
				trapEndPt - 1 - lrx, trapEndPt - lry, -plateau, 0,
				draw);
			break;
		case 3:
			polyominoImage.drawWallUnit(canvas,
				pmid + PolyominoImage.NUM_BITMAPS,
				cid, bid,
				fieldOffset.x, fieldOffset.y,
				uly, trapEndPt - 1 - ulx, 0, -trapBase,
				lry, trapEndPt - 1 - lrx, 0, -plateau,
				draw);
			break;
		default:
			System.err.println("trapazoid kinds range 0-3, got " +
				sideWall + ".");
		}
	}

	void drawTrapazoidBorder(Canvas canvas, int cid, int i, int j) {
		int ulx, uly, lrx, lry, trapBase, plateau, trapBaseWidth, trapEndPt, sideWall;
		int newI = i;

		trapBaseWidth =  diameter * boxSize + 1;
		trapEndPt = 3 * trapBaseWidth;
		sideWall = newI / diameter;
		newI -= (sideWall * diameter);
		ulx = (trapEndPt +  (2 * newI - diameter) * ((depth / 2 +
			2 * boxSize * (depth - j)) / depth + boxSize)) / 2;
		uly = (j * trapBaseWidth + depth / 2) / depth;
		lrx = (trapEndPt + (2  + 2 * newI  - diameter) * ((depth / 2 +
			2 * boxSize * (depth - j - 1)) / depth + boxSize)) / 2;
		lry = ((j + 1) * trapBaseWidth + depth / 2) / depth;
		trapBase = boxSize + ((depth - j) * 2 * boxSize + depth / 2) /
			depth;
		plateau = boxSize + ((depth - j - 1) * 2 * boxSize +
			depth / 2) / depth;

		switch (sideWall) {
		case 0:
			PolyominoImage.drawWallUnitBorder(canvas,
				cid,
				fieldOffset.x, fieldOffset.y,
				ulx + 1, uly, trapBase, 0,
				lrx + 1, lry, plateau, 0);
			break;
		case 1:
			PolyominoImage.drawWallUnitBorder(canvas,
				cid,
				fieldOffset.x, fieldOffset.y,
				trapEndPt - uly, ulx + 1, 0, trapBase,
				trapEndPt - lry, lrx + 1, 0, plateau);
			break;
		case 2:
			PolyominoImage.drawWallUnitBorder(canvas,
				cid,
				fieldOffset.x, fieldOffset.y,
				trapEndPt - 1 - ulx, trapEndPt - uly, -trapBase, 0,
				trapEndPt - 1 - lrx, trapEndPt - lry, -plateau, 0);
			break;
		case 3:
			PolyominoImage.drawWallUnitBorder(canvas,
				cid,
				fieldOffset.x, fieldOffset.y,
				uly, trapEndPt - 1 - ulx, 0, -trapBase,
				lry, trapEndPt - 1 - lrx, 0, -plateau);
			break;
		default:
			System.err.println("trapazoid kinds range 0-3, got " +
				sideWall + ".");
		}
	}

	void clearUnit(Canvas canvas, int i, int j) {
		PolyominoImage.clearUnits(canvas, new Colour(background),
			fieldOffset.x, fieldOffset.y,
			i, j, boxWidth, boxHeight);
		//.fillRect(i * boxSize + baseWidth + 1 + fieldOffset.x,
		//		j * boxSize + baseWidth + 1 + fieldOffset.y,
		//		boxSize, boxSize);
	}

	void drawBaseUnit(Canvas canvas, PolyominoImage polyominoImage,
			int pmid, Colour color, int i, int j) {
		polyominoImage.drawUnit(canvas, false, pmid, color,
			baseWidth + 1 + fieldOffset.x,
			baseWidth + 1 + fieldOffset.y,
			i, j, boxSize, boxSize);
	}


// ELIMINATE ME later?
	void drawBaseUnit(Canvas canvas, PolyominoImage polyominoImage,
			int pmid, int color, int i, int j) {
		polyominoImage.drawUnit(canvas, false, pmid, color,
			baseWidth + 1 + fieldOffset.x,
			baseWidth + 1 + fieldOffset.y,
			i, j, boxSize, boxSize);
	}

	int lc, cc;
	char c;

	char readChar() {
		int lcold = lc, ccold = cc;

		if (lc >= polyominoData.length)
			return ' ';
		if (cc == polyominoData[lc].length()) {
			lc++;
			cc = 0;
		} else
			cc++;
		if (ccold >= polyominoData[lcold].length())
			return ' ';
		return polyominoData[lcold].charAt(ccold);
	}

	int readInt() {
		boolean first = false;
		int loopcount = 0;
		int number;
		int MAX_LOOP = 100;

		while (!first) {
			c = readChar();
			if (c >= '0' && c <= '9')
				first = true;
			if (loopcount++ >= MAX_LOOP) {
				System.err.println(
					"Welltris: readInt1 infinite loop.");

			}
		}
		number = c - '0';
		first = false;
		loopcount = 0;
		while (!first) {
			int BASE = 10;
			c = readChar();
			if (c < '0' || c > '9')
				first = true;
			else
				number = BASE * number + (c - '0');
			if (loopcount++ >= MAX_LOOP) {
				System.err.println(
					"Welltris: readInt2 infinite loop.");
			}
		}
		return number;
	}

	void readPolyominoes() {
		int i, j, k, sq, polyomino, sum, start, n, size = 0;
		int height, diag, initHeight, toss = 0;
		int[] counter = new int [WelltrisInterface.MAX_UNITS -
			WelltrisInterface.MIN_UNITS + 1];
		int[][][] startCounter = new int [WelltrisInterface.MAX_UNITS -
			WelltrisInterface.MIN_UNITS + 1]
			[WelltrisInterface.MAX_MODES][WelltrisInterface.MAX_TYPES];

		lc = 0; cc = 0;
		for (sq = 0; sq <= WelltrisInterface.MAX_UNITS -
			 WelltrisInterface.MIN_UNITS; sq++) {
		  counter[sq] = 0;
		  for (polyomino = 0; polyomino <
			   WelltrisInterface.MAX_POLYOMINOES; polyomino++)
			for (j = 0; j < WelltrisInterface.MAX_UNITS; j++)
			  for (i = 0; i < WelltrisInterface.MAX_UNITS; i++)
			tris[sq].poly[polyomino].shape[j][i] = 0;
		  for (j = 0; j < WelltrisInterface.MAX_TYPES; j++) {
			for (polyomino = 0; polyomino <
			 WelltrisInterface.MAX_START_POLYOMINOES; polyomino++) {
			  tris[sq].mode[WelltrisInterface.NODIAG].start
			 [polyomino][j] = 0;
			  tris[sq].mode[WelltrisInterface.DIAGONAL].start
			 [polyomino][j] = 0;
			}
			startCounter[sq][WelltrisInterface.NODIAG][j] = 0;
			startCounter[sq][WelltrisInterface.DIAGONAL][j] = 0;
			tris[sq].mode[WelltrisInterface.NODIAG].number
			  [WelltrisInterface.NOMIX][j] = 0;
			tris[sq].mode[WelltrisInterface.DIAGONAL].number
			  [WelltrisInterface.NOMIX][j] = 0;
			tris[sq].mode[WelltrisInterface.NODIAG].number
			  [WelltrisInterface.MIXED][j] = 0;
			tris[sq].mode[WelltrisInterface.DIAGONAL].number
			  [WelltrisInterface.MIXED][j] = 0;
		  }
		  tris[sq].mode[WelltrisInterface.NODIAG].turnStyle =
			WelltrisInterface.NONE;
		  tris[sq].mode[WelltrisInterface.DIAGONAL].turnStyle =
			WelltrisInterface.NONE;
		}
		c = readChar();
		while (lc + 1 < polyominoData.length) {
		  if (c == '+') {
			k = readInt();
			sq = k - WelltrisInterface.MIN_UNITS;
			for (i = 0; i < WelltrisInterface.MAX_TYPES; i++) {
			  k = readInt();
			  tris[sq].mode[WelltrisInterface.NODIAG].number
			[WelltrisInterface.NOMIX][i] = k;
			}
			k = readInt();
			tris[sq].mode[WelltrisInterface.NODIAG].turnStyle = k;
			k = readInt();
			tris[sq].diagonalSwitch = k;
			if (tris[sq].diagonalSwitch != 0) {
			  for (i = 0; i < WelltrisInterface.MAX_TYPES; i++) {
			k = readInt();
			tris[sq].mode[WelltrisInterface.DIAGONAL].number
			  [WelltrisInterface.NOMIX][i] = k;
			  }
			  k = readInt();
			  tris[sq].mode[WelltrisInterface.DIAGONAL].turnStyle = k;
			}
			c = readChar();
		  }
		  if (c == '*') {
			size = readInt();
			c = readChar();
		  }
		  /* Useful for debugging Things */
/*
		  if (c == '~') {
			toss = readInt(&lc, &cc, &c);
		  }
*/
		  if (c == '#') {
			k = readInt();
			sq = k - WelltrisInterface.MIN_UNITS;
			n = readInt();
			if (tris[sq].diagonalSwitch != 0)
			  n = readInt();
			for (polyomino = 0; polyomino < n - toss; polyomino++) {
			  sum = polyomino + counter[sq];
			  /* This is only there to "read" input file */
			  readInt();
			  k = readInt();
			  tris[sq].poly[sum].rotation =
			k + counter[sq] - toss;
			  k = readInt();
			  tris[sq].poly[sum].reflection =
			k + counter[sq] - toss;
			  for (initHeight = WelltrisInterface.JUMPIN;
			   initHeight <= WelltrisInterface.GRADUAL; initHeight++) {
			height = readInt();
			if (initHeight == WelltrisInterface.JUMPIN)
			  tris[sq].poly[sum].startHeight[1] = height;
			else if (initHeight == WelltrisInterface.GRADUAL)
			  tris[sq].poly[sum].startHeight[0] = height;
			for (diag = WelltrisInterface.NODIAG;
				 diag <= tris[sq].diagonalSwitch; diag++) {
			  start = readInt();
			  if (initHeight == WelltrisInterface.JUMPIN) {
				if (sq == 0 ||
				tris[sq - 1].mode[diag].turnStyle ==
				WelltrisInterface.NONE) {
				  i = startCounter[sq][diag][WelltrisInterface.NONE];
				  tris[sq].mode[diag].start[i]
				 [WelltrisInterface.NONE] = sum;
				  startCounter[sq][diag][WelltrisInterface.NONE]++;
				}
				if ((sq == 0 ||
				tris[sq - 1].mode[diag].turnStyle <
				WelltrisInterface.ALL) && start !=
				WelltrisInterface.NONE) {
				  i = startCounter[sq][diag]
				[WelltrisInterface.NOREFL];
				  tris[sq].mode[diag].start[i]
				[WelltrisInterface.NOREFL] = sum;
				  startCounter[sq][diag][WelltrisInterface.NOREFL]++;
				}
				if (start == WelltrisInterface.ALL) {
				  i= startCounter[sq][diag][WelltrisInterface.ALL];
				  tris[sq].mode[diag].start[i]
				[WelltrisInterface.ALL] = sum;
				  startCounter[sq][diag][WelltrisInterface.ALL]++;
				}
			  }
			}
			  }
			  tris[sq].poly[sum].size = size;
			  for (j = 0; j < size; j++)
			for (i = 0; i < size; i++) {
			   k = readInt();
			   tris[sq].poly[sum].shape[j][i] = k;
			}
			}
			counter[sq] += n - toss;
			toss = 0;
			c = readChar();
		  }
		}
		for (i = 0; i <= WelltrisInterface.MAX_UNITS -
			 WelltrisInterface.MIN_UNITS; i++)
		  for (j = 0; j < WelltrisInterface.MAX_TYPES; j++)
			for (k = 0; k <= i; k++) {
			  tris[i].mode[WelltrisInterface.NODIAG].number
				[WelltrisInterface.MIXED][j] +=
			  tris[k].mode[WelltrisInterface.NODIAG].number
				[WelltrisInterface.NOMIX][j];
			  if (tris[i].diagonalSwitch != 0)
			/*since k & i are not independent*/
			tris[i].mode[WelltrisInterface.DIAGONAL].number
				[WelltrisInterface.MIXED][j] +=
			  tris[k].mode[WelltrisInterface.DIAGONAL].number
				[WelltrisInterface.NOMIX][j];
			}
	}

/* polyominoes for tetris & welltris */

/* Maximum number of squares was chosen to be 6 because if 7, it leaves a */
/*  hole: @@@ */
/*        @ @ */
/*        @@  */
/*  likewise, 3 was chosen for diagonal mode because if 4, it leaves a hole: */
/*   @  */
/*  @ @ */
/*   @  */

/* '+' Prelude part */
/*   1st number = number of squares in a polyomino */
/*   2nd number = number of distinct polyominoes & rotations & reflections */
/*   3rd number = number of distinct polyominoes & reflections */
/*   4th number = number of distinct polyominoes */
/*   5th number = 0 do not allow rotation and reflection */
/*                1 allow rotation but not reflection */
/*                2 allow rotation and reflection */
/*   6th number = diagonal mode to consider? 0 or 1 (if 0, no 7-10) */
/*   7th number = diagonal number of distinct polyominoes & rots & refls */
/*   8th number = diagonal number of distinct polyominoes & reflections */
/*   9th number = diagonal number of distinct polyominoes */
/*  10th number = 0 do not allow rotation and reflection for diagonal mode */
/*                1 allow rotation but not reflection for diagonal mode */
/*                2 allow rotation and reflection for diagonal mode */

/* '*' Main parts */
/*   1st number = n-> consider an n^2 square */
/* '#' polyomino group (follows a '*') */
/*   1st number = number of squares in a polyomino */
/*   2nd number = number of polyominoes (incl rot & refl) in a n^2 square */
/*   3rd number = if it is there, number of polyominoes (incl rot & refl) */
/*       in a n^2 square with diagonal mode */
/* The polyomino (follows a '#') */
/*   1st number = numbered polyomino (there for your benefit) */
/*   2nd number = rotation pointer (usually the next polyomino) */
/*   3rd number = reflection pointer */
/*   4th number = number of rows down to start */
/*   5th number = start number (0, 1, or 2) */
/*       0=> starter for no turns permitted (no reflections or rotations) */
/*       1=> starter for allow rotations and no turns permitted */
/*       2=> starter for reflections, rotations, and no turns permitted */
/*   Next number (if diagonal mode) = start number for diagonal mode */
/*   The proceeding 2 or 3 numbers are repeated for gradual appearance */
/*       Pieces either JUMPIN or appear GRADUALly */
/*   The last group of n^2 numbers is the actual polyomino */
/*       0=>     empty square */
/*       1..16=> used square */
/* '~' debug (precedes a '#') */
/*   1st number = number of polyominoes to skip */

String[] polyominoData = {

/* PRELUDE */
/* + Polyomino States(None Rot Refl) Style Diagonal [States(N R R) Style] */
/* + 1 1 1 1 0 1     1 1 1 0 */

"+ 2 2 1 1 0 1     4 2 2 0",
"+ 3 6 2 2 0 1     20 6 5 1",
"+ 4 19 7 5 1 0",
"+ 5 63 18 12 2 0",
"+ 6 216 60 35 2 0",

/* MAIN */
/* * X    (polyomino fills X^2 square) */
/* # Polyomino Number_of_Polyominoes_Normal [Number_of_Polyominoes_Diagonal] */
/* Ref_Num Ptr_Rot Ptr_Refl (Start_Location Start_N [Start_D])J&G Shape_Array */
/* * 1 */
/* # 1 1 */
/* 0 0 0  0 2 2 1 2 2   16 */
"* 2",
"# 2 2 4",
"0 1 0  1 2 2 1 0 0   0 0  4 1",
"1 0 1  0 0 0 1 2 2   0 2  0 8",

"2 3 3  0 0 2 1 0 2   0 16  16 0",
"3 2 2  0 0 0 1 0 0   16 0  0 16",
"# 3 4 4",
"0 1 3  0 2 2 1 2 2   6 1  8 0",
"1 2 2  0 0 0 1 0 0   2 0  12 1",
"2 3 1  0 0 0 1 0 0   0 2  4 9",
"3 0 0  0 0 0 1 0 0   4 3  0 8",
"# 4 1",
"0 0 0  0 2 1 2   6 3  12 9",
"* 3",
"# 3 2 16",
"0 1 0   1 2 2 1 0 0   0 0 0  4 5 1   0 0 0",
"1 0 1   0 0 0 2 2 2   0 2 0  0 10 0  0 8 0",

"2 3 3     0 0 2 2 0 2   16 0 0  0 16 0   0 0 16",
"3 2 2     0 0 0 2 0 0   0 0 16  0 16 0   16 0 0",
"4 5 8     1 0 2 2 0 0   0 0 0   0 4 1    16 0 0",
"5 6 11    0 0 0 2 0 2   0 2 0   0 8 0    0 0 16",
"6 7 10    1 0 0 2 0 0   0 0 0   0 0 16   4 1 0",
"7 4 9     0 0 0 2 0 0   0 16 0  0 0 2    0 0 8",
"8 9 4     1 0 1 2 0 0   0 0 0   4 1 0    0 0 16",
"9 10 7    0 0 0 2 0 0   0 0 16  0 2 0    0 8 0",
"10 11 6   1 0 0 2 0 0   0 0 0   16 0 0   0 4 1",
"11 8 5    0 0 0 2 0 1   0 0 2   0 0 8    0 16 0",
"12 13 12  1 0 2 2 0 0   0 0 0   16 0 16  0 16 0",
"13 14 15  0 0 0 2 0 2   0 16 0  0 0 16   0 16 0",
"14 15 14  1 0 0 2 0 0   0 0 0   0 16 0   16 0 16",
"15 12 13  0 0 0 2 0 0   0 0 16  0 16 0   0 0 16",
"# 4 16",
"0 1 4     1 2 2 0   0 0 0  6 5 1   8 0 0",
"1 2 7     0 0 2 0   0 2 0  0 10 0  0 12 1",
"2 3 6     1 0 2 0   0 0 0  0 0 2   4 5 9",
"3 0 5     0 0 2 2   0 4 3  0 0 10  0 0 8",
"4 5 0     1 1 2 0   0 0 0  4 5 3   0 0 8",
"5 6 3     0 0 2 1   0 6 1  0 10 0  0 8 0",
"6 7 2     1 0 2 0   0 0 0  2 0 0   12 5 1",
"7 4 1     0 0 2 0   0 0 2  0 0 10  0 4 9",
"8 9 8     1 2 2 0   0 0 0  4 7 1   0 8 0",
"9 10 11   0 0 2 2   0 2 0  0 14 1  0 8 0",
"10 11 10  1 0 2 0   0 0 0  0 2 0   4 13 1",
"11 8 9    0 0 2 0   0 0 2  0 4 11  0 0 8",
"12 13 14  1 2 2 0   0 0 0  4 3 0   0 12 1",
"13 12 15  0 0 2 2   0 0 2  0 6 9   0 8 0",
"14 15 12  1 1 2 0   0 0 0  0 6 1   4 9 0",
"15 14 13  0 0 2 1   0 2 0  0 12 3  0 0 8",
"# 5 37",
"0 1 0     1 2 2 0   0 0 0  6 5 3    8 0 8",
"1 2 3     0 0 2 2   0 6 1  0 10 0   0 12 1",
"2 3 2     1 0 2 0   0 0 0  2 0 2    12 5 9",
"3 0 1     0 0 2 0   0 4 3  0 0 10   0 4 9",
"4 5 8     1 2 2 0   0 0 0  6 7 1    12 9 0",
"5 6 11    0 0 2 0   0 2 0  0 14 3   0 12 9",
"6 7 10    1 0 2 0   0 0 0  0 6 3    4 13 9",
"7 4 9     0 0 2 2   0 6 3  0 12 11  0 0 8",
"8 9 4     1 1 2 0   0 0 0  4 7 3    0 12 9",
"9 10 7    0 0 2 1   0 6 3  0 14 9   0 8 0",
"10 11 6   1 0 2 0   0 0 0  6 3 0    12 13 1",
"11 8 5    0 0 2 0   0 0 2  0 6 11   0 12 9",
"12 13 12  0 2 2 2   4 7 1  0 10 0   0 8 0",
"13 14 15  0 0 2 0   2 0 0  14 5 1   8 0 0",
"14 15 14  0 0 2 0   0 2 0  0 10 0   4 13 1",
"15 12 13  0 0 2 0   0 0 2  4 5 11   0 0 8",
"16 17 19  0 2 2 2   6 5 1  10 0 0   8 0 0",
"17 18 18  0 0 2 0   2 0 0  10 0 0   12 5 1",
"18 19 17  0 0 2 0   0 0 2  0 0 10   4 5 9",
"19 16 16  0 0 2 0   4 5 3  0 0 10   0 0 8",
"20 21 22  0 2 2 0   2 0 0  12 5 3   0 0 8",
"21 20 23  0 0 2 2   0 6 1  0 10 0   4 9 0",
"22 23 20  0 1 2 0   0 0 2  6 5 9    8 0 0",
"23 22 21  0 0 2 1   4 3 0  0 10 0   0 12 1",
"24 25 25  0 2 2 2   4 3 0  0 12 3   0 0 8",
"25 26 24  0 0 2 0   0 6 1  6 9 0    8 0 0",
"26 27 27  0 0 2 0   2 0 0  12 3 0   0 12 1",
"27 24 26  0 0 2 0   0 0 2  0 6 9    4 9 0",
"28 29 32  0 2 2 0   0 2 0  6 13 1   8 0 0",
"29 30 35  0 0 2 0   0 2 0  4 11 0   0 12 1",
"30 31 34  0 0 2 0   0 0 2  4 7 9    0 8 0",
"31 28 33  0 0 2 2   4 3 0  0 14 1   0 8 0",
"32 33 28  0 1 2 0   0 2 0  4 13 3   0 0 8",
"33 34 31  0 0 2 1   0 6 1  4 11 0   0 8 0",
"34 35 30  0 0 2 0   2 0 0  12 7 1   0 8 0",
"35 32 29  0 0 2 0   0 2 0  0 14 1   4 9 0",
"36 36 36  0 2 2 2   0 2 0  4 15 1   0 8 0",
"# 6 46",
"0 1 4     0 2 2 0   6 5 3  10 0 8   8 0 0",
"1 2 7     0 0 2 0   6 1 0  10 0 0   12 5 1",
"2 3 6     0 0 2 0   0 0 2  2 0 10   12 5 9",
"3 0 5     0 0 2 2   4 5 3  0 0 10   0 4 9",
"4 5 0     0 1 2 0   6 5 3  8 0 10   0 0 8",
"5 6 3     0 0 2 1   6 5 1  10 0 0   12 1 0",
"6 7 2     0 0 2 0   2 0 0  10 0 2   12 5 9",
"7 4 1     0 0 2 0   0 4 3  0 0 10   4 5 9",
"8 9 12    0 2 2 2   6 7 1  12 11 0  0 8 0",
"9 10 15   0 0 2 0   2 0 0  14 7 1   12 9 0",
"10 11 14  0 0 2 0   0 2 0  0 14 3   4 13 9",
"11 8 13   0 0 2 0   0 6 3  4 13 11  0 0 8",
"12 13 8   0 1 2 1   4 7 3  0 14 9   0 8 0",
"13 14 11  0 0 2 0   6 3 0  14 13 1  8 0 0",
"14 15 10  0 0 2 0   0 2 0  6 11 0   12 13 1",
"15 12 9   0 0 2 0   0 0 2  4 7 11   0 12 9",
"16 17 20  0 2 2 0   6 3 0  12 13 3  0 0 8",
"17 18 23  0 0 2 0   0 6 1  6 11 0   12 9 0",
"18 19 22  0 0 2 0   2 0 0  12 7 3   0 12 9",
"19 16 21  0 0 2 2   0 6 3  0 14 9   4 9 0",
"20 21 16  0 1 2 0   0 6 3  6 13 9   8 0 0",
"21 22 19  0 0 2 1   6 3 0  12 11 0  0 12 1",
"22 23 18  0 0 2 0   0 0 2  6 7 9    12 9 0",
"23 20 17  0 0 2 0   4 3 0  0 14 3   0 12 9",
"24 25 28  0 2 2 2   4 7 1  0 10 0   4 9 0",
"25 26 31  0 0 2 0   2 0 0  14 5 3   8 0 8",
"26 27 30  0 0 2 0   0 6 1  0 10 0   4 13 1",
"27 24 29  0 0 2 0   2 0 2  12 5 11  0 0 8",
"28 29 24  0 1 2 1   4 7 1  0 10 0   0 12 1",
"29 30 27  0 0 2 0   2 0 2  14 5 9   8 0 0",
"30 31 26  0 0 2 0   4 3 0  0 10 0   4 13 1",
"31 28 25  0 0 2 0   0 0 2  6 5 11   8 0 8",
"32 33 32  0 2 2 0   2 0 2  12 7 9   0 8 0",
"33 34 35  0 0 2 2   4 3 0  0 14 1   4 9 0",
"34 35 34  0 0 2 0   0 2 0  6 13 3   8 0 8",
"35 32 33  0 0 2 0   0 6 1  4 11 0   0 12 1",
"36 37 39  0 2 2 2   6 7 1  14 9 0   8 0 0",
"37 38 38  0 0 2 0   2 0 0  14 3 0   12 13 1",
"38 39 37  0 0 2 0   0 0 2  0 6 11   4 13 9",
"39 36 36  0 0 2 0   4 7 3  0 12 11  0 0 8",
"40 41 43  0 2 2 2   6 3 0  12 15 1  0 8 0",
"41 42 42  0 0 2 0   0 2 0  6 15 1   12 9 0",
"42 43 41  0 0 2 0   0 2 0  4 15 3   0 12 9",
"43 40 40  0 0 2 0   0 6 3  4 15 9   0 8 0",
"44 45 44  1 2 2 0   0 0 0  6 7 3    12 13 9",
"45 44 45  0 0 2 2   0 6 3  0 14 11  0 12 9",
"* 4",
"# 4 2",
"0 1 0  2 2 2 0   0 0 0 0  0 0 0 0   4 5 5 1   0 0 0 0",
"1 0 1  0 0 3 2   0 0 2 0  0 0 10 0  0 0 10 0  0 0 8 0",
"# 5 24",
"0 1 4     1 2 2 0   0 0 0 0  6 5 5 1   8 0 0 0   0 0 0 0",
"1 2 7     0 0 3 0   0 2 0 0  0 10 0 0  0 10 0 0  0 12 1 0",
"2 3 6     1 0 2 0   0 0 0 0  0 0 0 2   4 5 5 9   0 0 0 0",
"3 0 5     0 0 3 2   0 4 3 0  0 0 10 0  0 0 10 0  0 0 8 0",
"4 5 0     1 1 2 0   0 0 0 0  4 5 5 3   0 0 0 8   0 0 0 0",
"5 6 3     0 0 3 1   0 6 1 0  0 10 0 0  0 10 0 0  0 8 0 0",
"6 7 2     1 0 2 0   0 0 0 0  2 0 0 0   12 5 5 1  0 0 0 0",
"7 4 1     0 0 3 0   0 0 2 0  0 0 10 0  0 0 10 0  0 4 9 0",
"8 9 12    1 2 2 0   0 0 0 0  4 7 5 1   0 8 0 0   0 0 0 0",
"9 10 15   0 0 3 0   0 2 0 0  0 10 0 0  0 14 1 0  0 8 0 0",
"10 11 14  1 0 2 0   0 0 0 0  0 0 2 0   4 5 13 1  0 0 0 0",
"11 8 13   0 0 3 2   0 0 2 0  0 4 11 0  0 0 10 0  0 0 8 0",
"12 13 8   1 1 2 0   0 0 0 0  4 5 7 1   0 0 8 0   0 0 0 0",
"13 14 11  0 0 3 1   0 2 0 0  0 14 1 0  0 10 0 0  0 8 0 0",
"14 15 10  1 0 2 0   0 0 0 0  0 2 0 0   4 13 5 1  0 0 0 0",
"15 12 9   0 0 3 0   0 0 2 0  0 0 10 0  0 4 11 0  0 0 8 0",
"16 17 20  1 2 2 0   0 0 0 0  4 5 3 0   0 0 12 1  0 0 0 0",
"17 18 23  0 0 3 2   0 0 2 0  0 6 9 0   0 10 0 0  0 8 0 0",
"18 19 22  1 0 2 0   0 0 0 0  4 3 0 0   0 12 5 1  0 0 0 0",
"19 16 21  0 0 3 0   0 0 2 0  0 0 10 0  0 6 9 0   0 8 0 0",
"20 21 16  1 1 2 0   0 0 0 0  0 6 5 1   4 9 0 0   0 0 0 0",
"21 22 19  0 0 3 0   0 2 0 0  0 10 0 0  0 12 3 0  0 0 8 0",
"22 23 18  1 0 2 0   0 0 0 0  0 0 6 1   4 5 9 0   0 0 0 0",
"23 20 17  0 0 3 1   0 2 0 0  0 12 3 0  0 0 10 0  0 0 8 0",
"# 6 136",
"0 1 4        1 2 3 0   0 0 0 0  6 5 5 1    10 0 0 0   8 0 0 0",
"1 2 7        0 0 3 0   0 2 0 0  0 10 0 0   0 10 0 0   0 12 5 1",
"2 3 6        1 0 3 0   0 0 0 0  0 0 0 2    0 0 0 10   4 5 5 9",
"3 0 5        0 0 3 2   0 4 5 3  0 0 0 10   0 0 0 10   0 0 0 8",
"4 5 0        1 1 3 0   0 0 0 0  4 5 5 3    0 0 0 10   0 0 0 8",
"5 6 3        0 0 3 1   0 6 5 1  0 10 0 0   0 10 0 0   0 8 0 0",
"6 7 2        1 0 3 0   0 0 0 0  2 0 0 0    10 0 0 0   12 5 5 1",
"7 4 1        0 0 3 0   0 0 0 2  0 0 0 10   0 0 0 10   0 4 5 9",
"8 9 12       1 2 2 0   0 0 0 0  6 7 5 1    12 9 0 0   0 0 0 0",
"9 10 15      0 0 3 0   0 2 0 0  0 10 0 0   0 14 3 0   0 12 9 0",
"10 11 14     1 0 2 0   0 0 0 0  0 0 6 3    4 5 13 9   0 0 0 0",
"11 8 13      0 0 3 2   0 6 3 0  0 12 11 0  0 0 10 0   0 0 8 0",
"12 13 8      1 1 2 0   0 0 0 0  4 5 7 3    0 0 12 9   0 0 0 0",
"13 14 11     0 0 3 1   0 6 3 0  0 14 9 0   0 10 0 0   0 8 0 0",
"14 15 10     1 0 2 0   0 0 0 0  6 3 0 0    12 13 5 1  0 0 0 0",
"15 12 9      0 0 3 0   0 0 2 0  0 0 10 0   0 6 11 0   0 12 9 0",
"16 17 20     1 2 2 0   0 0 0 0  6 5 7 1    8 0 8 0    0 0 0 0",
"17 18 23     0 0 3 0   0 2 0 0  0 14 1 0   0 10 0 0   0 12 1 0",
"18 19 22     1 0 2 0   0 0 0 0  0 2 0 2    4 13 5 9   0 0 0 0",
"19 16 21     0 0 3 2   0 4 3 0  0 0 10 0   0 4 11 0   0 0 8 0",
"20 21 16     1 1 2 0   0 0 0 0  4 7 5 3    0 8 0 8    0 0 0 0",
"21 22 19     0 0 3 1   0 6 1 0  0 10 0 0   0 14 1 0   0 8 0 0",
"22 23 18     1 0 2 0   0 0 0 0  2 0 2 0    12 5 13 1  0 0 0 0",
"23 20 17     0 0 3 0   0 0 2 0  0 4 11 0   0 0 10 0   0 4 9 0",
"24 25 28     1 2 3 0   0 0 0 0  4 7 5 1    0 10 0 0   0 8 0 0",
"25 26 31     0 0 3 0   0 2 0 0  0 10 0 0   0 14 5 1   0 8 0 0",
"26 27 30     1 0 3 0   0 0 0 0  0 0 2 0    0 0 10 0   4 5 13 1",
"27 24 29     0 0 3 2   0 0 0 2  0 4 5 11   0 0 0 10   0 0 0 8",
"28 29 24     1 1 3 0   0 0 0 0  4 5 7 1    0 0 10 0   0 0 8 0",
"29 30 27     0 0 3 1   0 2 0 0  0 14 5 1   0 10 0 0   0 8 0 0",
"30 31 26     1 0 3 0   0 0 0 0  0 2 0 0    0 10 0 0   4 13 5 1",
"31 28 25     0 0 3 0   0 0 0 2  0 0 0 10   0 4 5 11   0 0 0 8",
"32 33 36     1 2 3 0   0 0 0 0  0 2 0 0    6 13 5 1   8 0 0 0",
"33 34 39     0 0 3 0   0 0 2 0  0 0 10 0   0 4 11 0   0 0 12 1",
"34 35 38     1 0 3 0   0 0 0 0  0 0 0 2    4 5 7 9    0 0 8 0",
"35 32 37     0 0 3 2   0 4 3 0  0 0 14 1   0 0 10 0   0 0 8 0",
"36 37 32     1 1 3 0   0 0 0 0  0 0 2 0    4 5 13 3   0 0 0 8",
"37 38 35     0 0 3 1   0 0 6 1  0 4 11 0   0 0 10 0   0 0 8 0",
"38 39 34     1 0 3 0   0 0 0 0  2 0 0 0    12 7 5 1   0 8 0 0",
"39 36 33     0 0 3 0   0 0 2 0  0 0 10 0   0 0 14 1   0 4 9 0",
"40 41 44     1 2 3 0   0 0 0 0  0 0 2 0    6 5 13 1   8 0 0 0",
"41 42 47     0 0 3 0   0 0 2 0  0 4 11 0   0 0 10 0   0 0 12 1",
"42 43 46     1 0 3 0   0 0 0 0  0 0 0 2    4 7 5 9    0 8 0 0",
"43 40 45     0 0 3 2   0 4 3 0  0 0 10 0   0 0 14 1   0 0 8 0",
"44 45 40     1 1 3 0   0 0 0 0  0 2 0 0    4 13 5 3   0 0 0 8",
"45 46 43     0 0 3 1   0 0 6 1  0 0 10 0   0 4 11 0   0 0 8 0",
"46 47 42     1 0 3 0   0 0 0 0  2 0 0 0    12 5 7 1   0 0 8 0",
"47 44 41     0 0 3 0   0 0 2 0  0 0 14 1   0 0 10 0   0 4 9 0",
"48 49 52     1 2 2 0   0 0 0 0  6 5 3 0    8 0 12 1   0 0 0 0",
"49 50 55     0 0 3 0   0 0 2 0  0 6 9 0    0 10 0 0   0 12 1 0",
"50 51 54     1 0 2 0   0 0 0 0  4 3 0 2    0 12 5 9   0 0 0 0",
"51 48 53     0 0 3 2   0 4 3 0  0 0 10 0   0 6 9 0    0 8 0 0",
"52 53 48     1 1 2 0   0 0 0 0  0 6 5 3    4 9 0 8    0 0 0 0",
"53 54 51     0 0 3 1   0 6 1 0  0 10 0 0   0 12 3 0   0 0 8 0",
"54 55 50     1 0 2 0   0 0 0 0  2 0 6 1    12 5 9 0   0 0 0 0",
"55 52 49     0 0 3 0   0 2 0 0  0 12 3 0   0 0 10 0   0 4 9 0",
"56 57 60     1 2 3 0   0 0 0 0  4 5 3 0    0 0 10 0   0 0 12 1",
"57 58 63     0 0 3 2   0 0 0 2  0 6 5 9    0 10 0 0   0 8 0 0",
"58 59 62     1 0 3 0   0 0 0 0  4 3 0 0    0 10 0 0   0 12 5 1",
"59 56 61     0 0 3 0   0 0 0 2  0 0 0 10   0 6 5 9    0 8 0 0",
"60 61 56     1 1 3 0   0 0 0 0  0 6 5 1    0 10 0 0   4 9 0 0",
"61 62 59     0 0 3 0   0 2 0 0  0 10 0 0   0 12 5 3   0 0 0 8",
"62 63 58     1 0 3 0   0 0 0 0  0 0 6 1    0 0 10 0   4 5 9 0",
"63 60 57     0 0 3 1   0 2 0 0  0 12 5 3   0 0 0 10   0 0 0 8",
"64 65 68     1 2 3 0   0 0 0 0  4 5 3 0    0 0 14 1   0 0 8 0",
"65 66 71     0 0 3 2   0 0 2 0  0 6 13 1   0 10 0 0   0 8 0 0",
"66 67 70     1 0 3 0   0 0 0 0  0 2 0 0    4 11 0 0   0 12 5 1",
"67 64 69     0 0 3 0   0 0 0 2  0 0 0 10   0 4 7 9    0 0 8 0",
"68 69 64     1 1 3 0   0 0 0 0  0 6 5 1    4 11 0 0   0 8 0 0",
"69 70 67     0 0 3 0   0 2 0 0  0 10 0 0   0 12 7 1   0 0 8 0",
"70 71 66     1 0 3 0   0 0 0 0  0 0 2 0    0 0 14 1   4 5 9 0",
"71 68 65     0 0 3 1   0 0 2 0  0 4 13 3   0 0 0 10   0 0 0 8",
"72 73 76     1 2 3 0   0 0 0 0  4 5 3 0    0 0 12 3   0 0 0 8",
"73 74 79     0 0 3 2   0 0 6 1  0 6 9 0    0 10 0 0   0 8 0 0",
"74 75 78     1 0 3 0   0 0 0 0  2 0 0 0    12 3 0 0   0 12 5 1",
"75 72 77     0 0 3 0   0 0 0 2  0 0 0 10   0 0 6 9    0 4 9 0",
"76 77 72     1 1 3 0   0 0 0 0  0 6 5 1    6 9 0 0    8 0 0 0",
"77 78 75     0 0 3 0   0 2 0 0  0 10 0 0   0 12 3 0   0 0 12 1",
"78 79 74     1 0 3 0   0 0 0 0  0 0 0 2    0 0 6 9    4 5 9 0",
"79 76 73     0 0 3 1   0 4 3 0  0 0 12 3   0 0 0 10   0 0 0 8",
"80 81 84     1 2 3 0   0 0 0 0  4 3 0 0    0 14 5 1   0 8 0 0",
"81 82 87     0 0 3 0   0 0 2 0  0 0 10 0   0 6 13 1   0 8 0 0",
"82 83 86     1 0 3 0   0 0 0 0  0 0 2 0    4 5 11 0   0 0 12 1",
"83 80 85     0 0 3 2   0 0 0 2  0 4 7 9    0 0 10 0   0 0 8 0",
"84 85 80     1 1 3 0   0 0 0 0  0 0 6 1    4 5 11 0   0 0 8 0",
"85 86 83     0 0 3 1   0 2 0 0  0 12 7 1   0 0 10 0   0 0 8 0",
"86 87 82     1 0 3 0   0 0 0 0  0 2 0 0    0 14 5 1   4 9 0 0",
"87 84 81     0 0 3 0   0 0 2 0  0 0 10 0   0 4 13 3   0 0 0 8",
"88 89 92     1 2 3 0   0 0 0 0  4 3 0 0    0 12 7 1   0 0 8 0",
"89 90 95     0 0 3 0   0 0 2 0  0 0 14 1   0 6 9 0    0 8 0 0",
"90 91 94     1 0 3 0   0 0 0 0  0 2 0 0    4 13 3 0   0 0 12 1",
"91 88 93     0 0 3 2   0 0 0 2  0 0 6 9    0 4 11 0   0 0 8 0",
"92 93 88     1 1 3 0   0 0 0 0  0 0 6 1    4 7 9 0    0 8 0 0",
"93 94 91     0 0 3 1   0 2 0 0  0 12 3 0   0 0 14 1   0 0 8 0",
"94 95 90     1 0 3 0   0 0 0 0  0 0 2 0    0 6 13 1   4 9 0 0",
"95 92 89     0 0 3 0   0 0 2 0  0 4 11 0   0 0 12 3   0 0 0 8",
"96 97 100    1 2 3 0   0 0 0 0  4 3 0 0    0 12 5 3   0 0 0 8",
"97 98 103    0 0 3 2   0 0 6 1  0 0 10 0   0 6 9 0    0 8 0 0",
"98 99 102    1 0 3 0   0 0 0 0  2 0 0 0    12 5 3 0   0 0 12 1",
"99 96 101    0 0 3 0   0 0 0 2  0 0 6 9    0 0 10 0   0 4 9 0",
"100 101 96   1 1 3 0   0 0 0 0  0 0 6 1    6 5 9 0    8 0 0 0",
"101 102 99   0 0 3 0   0 2 0 0  0 12 3 0   0 0 10 0   0 0 12 1",
"102 103 98   1 0 3 0   0 0 0 0  0 0 0 2    0 6 5 9    4 9 0 0",
"103 100 97   0 0 3 1   0 4 3 0  0 0 10 0   0 0 12 3   0 0 0 8",
"104 105 104  1 2 2 0   0 0 0 0  4 7 7 1    0 12 9 0   0 0 0 0",
"105 106 107  0 0 3 2   0 2 0 0  0 14 3 0   0 14 9 0   0 8 0 0",
"106 107 106  1 0 2 0   0 0 0 0  0 6 3 0    4 13 13 1  0 0 0 0",
"107 104 105  0 0 3 0   0 0 2 0  0 6 11 0   0 12 11 0  0 0 8 0",
"108 109 108  1 2 2 0   0 0 0 0  6 5 5 3    8 0 0 8    0 0 0 0",
"109 110 111  0 0 3 2   0 6 1 0  0 10 0 0   0 10 0 0   0 12 1 0",
"110 111 110  1 0 2 0   0 0 0 0  2 0 0 2    12 5 5 9   0 0 0 0",
"111 108 109  0 0 3 0   0 4 3 0  0 0 10 0   0 0 10 0   0 4 9 0",
"112 113 114  1 2 2 0   0 0 0 0  4 7 3 0    0 12 13 1  0 0 0 0",
"113 112 115  0 0 3 2   0 0 2 0  0 6 11 0   0 14 9 0   0 8 0 0",
"114 115 112  1 1 2 0   0 0 0 0  0 6 7 1    4 13 9 0   0 0 0 0",
"115 114 113  0 0 3 1   0 2 0 0  0 14 3 0   0 12 11 0  0 0 8 0",
"116 117 118  1 2 3 0   0 0 0 0  4 3 0 0    0 12 3 0   0 0 12 1",
"117 116 119  0 0 3 2   0 0 0 2  0 0 6 9    0 6 9 0    0 8 0 0",
"118 119 116  1 1 3 0   0 0 0 0  0 0 6 1    0 6 9 0    4 9 0 0",
"119 118 117  0 0 3 1   0 2 0 0  0 12 3 0   0 0 12 3   0 0 0 8",
"120 121 122  1 2 3 0   0 0 0 0  2 0 0 0    14 5 5 1   8 0 0 0",
"121 122 121  0 0 3 0   0 0 2 0  0 0 10 0   0 0 10 0   0 4 13 1",
"122 123 120  1 0 3 0   0 0 0 0  0 0 0 2    4 5 5 11   0 0 0 8",
"123 120 123  0 0 3 2   0 4 7 1  0 0 10 0   0 0 10 0   0 0 8 0",
"124 125 126  1 2 3 0   0 0 0 0  0 2 0 0    4 15 5 1   0 8 0 0",
"125 126 125  0 0 3 0   0 0 2 0  0 0 10 0   0 4 15 1   0 0 8 0",
"126 127 124  1 0 3 0   0 0 0 0  0 0 2 0    4 5 15 1   0 0 8 0",
"127 124 127  0 0 3 2   0 0 2 0  0 4 15 1   0 0 10 0   0 0 8 0",
"128 129 130  1 2 3 0   0 0 0 0  0 2 0 0    4 13 7 1   0 0 8 0",
"129 128 131  0 0 3 2   0 0 2 0  0 0 14 1   0 4 11 0   0 0 8 0",
"130 131 128  1 1 3 0   0 0 0 0  0 0 2 0    4 7 13 1   0 8 0 0",
"131 128 129  0 0 3 1   0 0 2 0  0 4 11 0   0 0 14 1   0 0 8 0",
"132 133 134  1 2 3 0   0 0 0 0  2 0 0 0    12 5 5 3   0 0 0 8",
"133 132 135  0 0 3 2   0 0 6 1  0 0 10 0   0 0 10 0   0 4 9 0",
"134 135 132  1 1 3 0   0 0 0 0  0 0 0 2    6 5 5 9    8 0 0 0",
"135 134 133  0 0 3 1   0 4 3 0  0 0 10 0   0 0 10 0   0 0 12 1",
"* 5",
"# 5 2",
"0 1 0  2 2 2 0   0 0 0 0 0  0 0 0 0 0   4 5 5 5 1   0 0 0 0 0   0 0 0 0 0",
"1 0 1  0 0 4 2   0 0 2 0 0  0 0 10 0 0  0 0 10 0 0  0 0 10 0 0  0 0 8 0 0",
"# 6 32",
"0 1 4     2 2 3 0   0 0 0 0 0  0 0 0 0 0   6 5 5 5 1   8 0 0 0 0   0 0 0 0 0",
"1 2 7     0 0 4 0   0 0 2 0 0  0 0 10 0 0  0 0 10 0 0  0 0 10 0 0  0 0 12 1 0",
"2 3 6     2 0 3 0   0 0 0 0 0  0 0 0 0 0   0 0 0 0 2   4 5 5 5 9   0 0 0 0 0",
"3 0 5     0 0 4 2   0 0 4 3 0  0 0 0 10 0  0 0 0 10 0  0 0 0 10 0  0 0 0 8 0",
"4 5 0     2 1 3 0   0 0 0 0 0  0 0 0 0 0   4 5 5 5 3   0 0 0 0 8   0 0 0 0 0",
"5 6 3     0 0 4 1   0 0 6 1 0  0 0 10 0 0  0 0 10 0 0  0 0 10 0 0  0 0 8 0 0",
"6 7 2     2 0 3 0   0 0 0 0 0  0 0 0 0 0   2 0 0 0 0   12 5 5 5 1  0 0 0 0 0",
"7 4 1     0 0 4 0   0 0 0 2 0  0 0 0 10 0  0 0 0 10 0  0 0 0 10 0  0 0 4 9 0",
"8 9 12    2 2 3 0   0 0 0 0 0  0 0 0 0 0   4 7 5 5 1   0 8 0 0 0   0 0 0 0 0",
"9 10 15   0 0 4 0   0 0 2 0 0  0 0 10 0 0  0 0 10 0 0  0 0 14 1 0  0 0 8 0 0",
"10 11 14  2 0 3 0   0 0 0 0 0  0 0 0 0 0   0 0 0 2 0   4 5 5 13 1  0 0 0 0 0",
"11 8 13   0 0 4 2   0 0 0 2 0  0 0 4 11 0  0 0 0 10 0  0 0 0 10 0  0 0 0 8 0",
"12 13 8   2 1 3 0   0 0 0 0 0  0 0 0 0 0   4 5 5 7 1   0 0 0 8 0   0 0 0 0 0",
"13 14 11  0 0 4 1   0 0 2 0 0  0 0 14 1 0  0 0 10 0 0  0 0 10 0 0  0 0 8 0 0",
"14 15 10  2 0 3 0   0 0 0 0 0  0 0 0 0 0   0 2 0 0 0   4 13 5 5 1  0 0 0 0 0",
"15 12 9   0 0 4 0   0 0 0 2 0  0 0 0 10 0  0 0 0 10 0  0 0 4 11 0  0 0 0 8 0",
"16 17 20  2 2 3 0   0 0 0 0 0  0 0 0 0 0   4 5 5 3 0   0 0 0 12 1  0 0 0 0 0",
"17 18 23  0 0 4 2   0 0 0 2 0  0 0 6 9 0   0 0 10 0 0  0 0 10 0 0  0 0 8 0 0",
"18 19 22  2 0 3 0   0 0 0 0 0  0 0 0 0 0   4 3 0 0 0   0 12 5 5 1  0 0 0 0 0",
"19 16 21  0 0 4 0   0 0 0 2 0  0 0 0 10 0  0 0 0 10 0  0 0 6 9 0   0 0 8 0 0",
"20 21 16  2 1 3 0   0 0 0 0 0  0 0 0 0 0   0 6 5 5 1   4 9 0 0 0   0 0 0 0 0",
"21 22 19  0 0 4 0   0 0 2 0 0  0 0 10 0 0  0 0 10 0 0  0 0 12 3 0  0 0 0 8 0",
"22 23 18  2 0 3 0   0 0 0 0 0  0 0 0 0 0   0 0 0 6 1   4 5 5 9 0   0 0 0 0 0",
"23 20 17  0 0 4 1   0 0 2 0 0  0 0 12 3 0  0 0 0 10 0  0 0 0 10 0  0 0 0 8 0",
"24 25 24  2 2 3 0   0 0 0 0 0  0 0 0 0 0   4 5 7 5 1   0 0 8 0 0   0 0 0 0 0",
"25 26 27  0 0 4 2   0 0 2 0 0  0 0 10 0 0  0 0 14 1 0  0 0 10 0 0  0 0 8 0 0",
"26 27 26  2 0 3 0   0 0 0 0 0  0 0 0 0 0   0 0 2 0 0   4 5 13 5 1  0 0 0 0 0",
"27 24 25  0 0 4 0   0 0 0 2 0  0 0 0 10 0  0 0 4 11 0  0 0 0 10 0  0 0 0 8 0",
"28 29 30  2 2 3 0   0 0 0 0 0  0 0 0 0 0   4 5 3 0 0   0 0 12 5 1  0 0 0 0 0",
"29 28 31  0 0 4 2   0 0 0 2 0  0 0 0 10 0  0 0 6 9 0   0 0 10 0 0  0 0 8 0 0",
"30 31 28  2 1 3 0   0 0 0 0 0  0 0 0 0 0   0 0 6 5 1   4 5 9 0 0   0 0 0 0 0",
"31 30 29  0 0 4 1   0 0 2 0 0  0 0 10 0 0  0 0 12 3 0  0 0 0 10 0  0 0 0 8 0",
"* 6",
"# 6 2",
"0 1 0  3 2 3 0   0 0 0 0 0 0   0 0 0 0 0 0   0 0 0 0 0 0",
"                 4 5 5 5 5 1   0 0 0 0 0 0   0 0 0 0 0 0",
"1 0 1  0 0 5 2   0 0 0 2 0 0   0 0 0 10 0 0  0 0 0 10 0 0",
"                 0 0 0 10 0 0  0 0 0 10 0 0  0 0 0 8 0 0"};
}

class Field {
	int pmid, cid; // pixmaps and colors

	public Field() {
		pmid = -1;
		cid = 0;
	}
}

class Spec {
	volatile int units;
	volatile boolean diagonal, mixed, bonus;

	public Spec(int units,
			boolean diagonal, boolean mixed, boolean bonus) {
		this.units = units;
		this.diagonal = diagonal;
		this.mixed = mixed;
		this.bonus = bonus;
	}

	public Spec(int units) {
		this.units = units;
		this.diagonal = this.mixed = this.bonus = false;
	}

	void copyValues(Spec nextSpec) {
		units = nextSpec.units;
		diagonal = nextSpec.diagonal;
		mixed = nextSpec.mixed;
		bonus = nextSpec.bonus;
	}

	public void setDiagonal(boolean diagonal) {
		this.diagonal = diagonal;
	}

	public void setMixed(boolean mixed) {
		this.mixed = mixed;
	}

	public void setBonus(boolean bonus) {
		this.bonus = bonus;
	}

	public void setUnits(int units) {
		this.units = units;
	}
}

