package com.sillycycle.bagleyd.welltris;

/*
 * @(#)WelltrisInterface.java
 *
 * Copyright 1993 - 2019  David A. Bagley, bagleyd AT verizon.net
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995 Q. Alex Zhao, azhao AT cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * The <code>WelltrisInterface</code> class contains most of the
 * constants used by the <code>WelltrisApp</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/altris.html
 */

public class WelltrisInterface {
	static final String SCOREFILE = "welltris.score";
	static final int NUM_FLASHES = 4;

	/* static final int BONUSIN = 25000;
	static final int BONUSOUT = 1000;
	static final int SAMPLERATE = 5000000; */

	static final int MAX_SCORES = 3;
	static final int SHOWSCORES = 15;

	static final int EMPTY = -1;
	static final int DEFAULT_BOXSIZE = 24;
	static final int NEXT_BOXSIZE = 16;
	static final int MIN_LEVELS = 1;
	static final int MAX_LEVELS = 20;
	static final int MAX_SIDES = 4;
	static final int NODIAG = 0;   /* Normal, ie no diagonals */
	static final int DIAGONAL = 1; /* Normal and diagonal */
	static final int NOMIX = 0;    /* Do not mix lowered number units */
	static final int MIXED = 1;
	static final int MAX_MODES = 2;
	static final int NONE = 0;   /* No rotation or reflection */
	static final int NOREFL = 1; /* Normal, no reflection allow rotation */
	static final int ALL = 2;    /* Allow reflection or rotation */
	static final int MAX_TYPES = 3;
	static final int JUMPIN = 0;
	static final int GRADUAL = 1;
	static final int DEFAULT_UNITS = 4;
	static final int MIN_UNITS = 2; /* Why not 1 you say? */
	static final int MAX_UNITS = 6;
	static final int DEFAULT_DIAGONALUNITS = 3;
	static final int MIN_DIAGONALUNITS = MIN_UNITS;
	static final int MAX_DIAGONALUNITS = 3;
	static final int MAX_POLYOMINOES = 216;
	static final int MAX_START_POLYOMINOES = 55;
	static final int DEFAULT_WIDTH = 8;
	static final int DEFAULT_DEPTH = 12;
	static final int MIN_WIDTH = MAX_UNITS;
	static final int MAX_WIDTH = 2 * DEFAULT_WIDTH;
	static final int DEFAULT_FILL = 0;
	static final int MAX_FILL = DEFAULT_WIDTH;
	static final int DEFAULT_LEVEL = 3;

	static final String MSG_PAUSED = "Paused";
	static final String MSG_END = "Game Over";
	//static final String TITLE = "Welltris";
	static final String TITLE = "Wellt\u044fis";
	public enum Icons {X16, X22, X24, X32, X48, X64}
	static final String[] ICONS = {"icons/16x16/welltris.png",
		"icons/22x22/welltris.png",
		"icons/24x24/welltris.png",
		"icons/32x32/welltris.png",
		"icons/48x48/welltris.png",
		"icons/64x64/welltris.png"};
	public enum Sounds {BUMP, DRIP}
	static final String[] SOUNDS = {"sounds/bump.wav",
		"sounds/drip.wav"};

	static boolean legal(int units,
			boolean diagonal, boolean mixed, boolean bonus) {
		return ((units >= MIN_UNITS + ((mixed) ? 1 : 0)) &&
			(units <= ((diagonal) ? MAX_DIAGONALUNITS :
			MAX_UNITS) - ((bonus) ? 1 : 0)));
	}
	static synchronized int threshold(int x) {
		return ((x + 1) * 10 + x);
	}

	static final int FALL = 1, DROP = 2, MOVE_LEFT = 3, MOVE_RIGHT = 4,
		ROTATE = 5, REFLECT = 6;
}
